/*
   Copyright (C) 2005, Intel Corporation.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


 *(C) Copyright 2006 Marvell International Ltd.
 * All Rights Reserved
 */

/*****************************************************************************
 * Abstract:
 *	Contains all hardware related functions for OV7673
 *
 * Notes:Only valid for processor code named Monahans.
 *****************************************************************************/
#include <linux/init.h>
#include <linux/module.h>
#include "camera.h"
#include "ov7673_hw.h"
#include <linux/delay.h>

#include <asm/arch/mfp.h>
#include <asm/arch/gpio.h>
#include <asm/hardware.h>

u8 bg = 0x80;
u8 rg = 0x24;

extern int platform_id(void);

/*****************************************************************************
 *  Attention: This is platform related!
 *****************************************************************************/

/*****************************************************************************
 *  Register Settings
 *****************************************************************************/
const static u8 gRAW_CIF_INITIALIZE[] =
{
	0x12, 0x80,
	0x11, 0x81,
	0x9e, 0x29,
	0x3b, 0x02,
	0x13, 0xf2,
	0x00, 0x00,
	0x10, 0x00,
	0x01, 0x80,
	0x02, 0x80,
	0x13, 0xf7,
	0x12, 0x25,
	0x04, 0x00,
	0x0c, 0x04,
	0x18, 0x7c,
	0x17, 0x24,
	0x32, 0xbf,
	0x1a, 0x4a,
	0x19, 0x02,
	0x03, 0x00,
	0x0e, 0x84,
	0x0f, 0x62,
	0x15, 0x02,
	0x1b, 0x01,
	0x1e, 0x01,
	0x29, 0x3c,
	0x33, 0x02,
	0x34, 0x00,
	0x35, 0x84,
	0x36, 0x03,
	0x38, 0x13,
	0x39, 0x43,
	0x3c, 0x28,
	0x3d, 0x19,
	0x40, 0xc1,
	0x69, 0x80,
	0x6b, 0x0d,
	0xa1, 0x08,
	0x8b, 0xcc,
	0x8c, 0xcc,
	0x8d, 0xcf,
	0x14, 0x2e,
	0x25, 0x58,
	0x24, 0x68,
	OV7673_REGEND, 0x00	    /* End of list delimiter */
};

const static u8 gRAW_QCIF_INITIALIZE[] =
{
	0x12, 0x80,
	0x11, 0x81,
	0x9e, 0x29,
	0x3b, 0x02,
	0x13, 0xf2,
	0x00, 0x00,
	0x10, 0x00,
	0x01, 0x80,
	0x02, 0x80,
	0x13, 0xf7,
	0x12, 0x0d,
	0x04, 0x04,
	0x0c, 0x0c,
	0x18, 0x7c,
	0x17, 0x24,
	0x32, 0xbf,
	0x1a, 0x4a,
	0x19, 0x02,
	0x03, 0x00,
	0x0e, 0x84,
	0x0f, 0x62,
	0x15, 0x02,
	0x1b, 0x01,
	0x1e, 0x01,
	0x29, 0x3c,
	0x33, 0x02,
	0x34, 0x00,
	0x35, 0x84,
	0x36, 0x03,
	0x38, 0x13,
	0x39, 0x43,
	0x3c, 0x28,
	0x3d, 0x19,
	0x40, 0xc1,
	0x69, 0x80,
	0x6b, 0x0d,
	0xa1, 0x08,
	0x8b, 0xcc,
	0x8c, 0xcc,
	0x8d, 0xcf,
	0x14, 0x2e,
	0x25, 0x58,
	0x24, 0x68,
	0x2a, 0x30,
	OV7673_REGEND, 0x00	    /* End of list delimiter */
};

const static u8 gRAW_QQCIF_INITIALIZE[] =
{
	0x12, 0x80,
	0x11, 0x81,
	0x9e, 0x29,
	0x3b, 0x02,
	0x13, 0xf2,
	0x00, 0x00,
	0x10, 0x00,
	0x01, 0x80,
	0x02, 0x80,
	0x13, 0xf7,
	0x12, 0x0d,
	0x04, 0x24,
	0x0c, 0x0c,
	0x18, 0x7c,
	0x17, 0x24,
	0x32, 0xbf,
	0x1a, 0x4a,
	0x19, 0x02,
	0x03, 0x00,
	0x0e, 0x84,
	0x0f, 0x62,
	0x15, 0x02,
	0x1b, 0x01,
	0x1e, 0x01,
	0x29, 0x3c,
	0x33, 0x02,
	0x34, 0x00,
	0x35, 0x84,
	0x36, 0x03,
	0x38, 0x13,
	0x39, 0x43,
	0x3c, 0x28,
	0x3d, 0x19,
	0x40, 0xc1,
	0x69, 0x80,
	0x6b, 0x0f,
	0xa1, 0x08,
	0x8b, 0xcc,
	0x8c, 0xcc,
	0x8d, 0xcf,
	0x14, 0x2e,
	0x25, 0x58,
	0x24, 0x68,
	OV7673_REGEND, 0x00	    /* End of list delimiter */
};

const static u8 gRAW_QQVGA_INITIALIZE[] =
{
	0x12, 0x80,
	0x11, 0x83,
	0x9e, 0x29,
	0x3b, 0x02,
	0x13, 0xf2,
	0x00, 0x00,
	0x10, 0x00,
	0x01, 0x80,
	0x02, 0x80,
	0x13, 0xf7,
	0x12, 0x15,
	0x04, 0x24,
	0x0c, 0x0c,
	0x18, 0x4a,
	0x17, 0x22,
	0x32, 0x92,
	0x19, 0x02,
	0x1a, 0x7a,
	0x03, 0x00,
	0x0e, 0x84,
	0x0f, 0x62,
	0x15, 0x02,
	0x1b, 0x01,
	0x1e, 0x01,
	0x29, 0x3c,
	0x34, 0x07,
	0x35, 0x84,
	0x36, 0x00,
	0x38, 0x13,
	0x39, 0x43,
	0x3c, 0x68,
	0x3d, 0x19,
	0x40, 0xc1,
	0x69, 0x80,
	0x6b, 0x0a,
	0xa1, 0x08,
	0x8b, 0xcc,
	0x8c, 0xcc,
	0x8d, 0xcf,
	0x14, 0x4e,
	0x25, 0x58,
	0x24, 0x68,
	0x92, 0xd8,
	0x93, 0x01,
	OV7673_REGEND, 0x00	    /* End of list delimiter */
};

const static u8 gRAW_QVGA_INITIALIZE[] =
{
	0x12, 0x80,
	0x11, 0x83,
	0x9e, 0x29,
	0x3b, 0x02,
	0x13, 0xf2,
	0x00, 0x00,
	0x10, 0x00,
	0x01, 0x80,
	0x02, 0x80,
	0x13, 0xf7,
	0x12, 0x15,
	0x04, 0x00,
	0x0c, 0x04,
	0x18, 0x4a,
	0x17, 0x22,
	0x32, 0x89,
	0x19, 0x02,
	0x1a, 0x7a,
	0x03, 0x05,
	0x0e, 0x84,
	0x0f, 0x62,
	0x15, 0x02,
	0x1b, 0x01,
	0x1e, 0x01,
	0x29, 0x3c,
	0x33, 0x00,
	0x34, 0x07,
	0x35, 0x84,
	0x36, 0x00,
	0x38, 0x13,
	0x39, 0x43,
	0x3c, 0x68,
	0x3d, 0x19,
	0x40, 0xc1,
	0x69, 0x80,
	0x6b, 0x0a,
	0xa1, 0x08,
	0x8b, 0xcc,
	0x8c, 0xcc,
	0x8d, 0xcf,
	0x14, 0x2e,
	0x25, 0x58,
	0x24, 0x68,
	0x92, 0x48,
	0x93, 0x00,
	OV7673_REGEND, 0x00	    /* End of list delimiter */
};

const static u8 gRAW_VGA_INITIALIZE[] =
{
	0x12, 0x80,
	0x11, 0x81,
	0x9e, 0x3f,
	0x3b, 0x02,
	0x13, 0xf2,
	0x00, 0x00,
	0x10, 0x00,
	0x01, 0x80,
	0x02, 0x80,
	0x13, 0xf7,
	0x12, 0x05,
	0x04, 0x00,
	0x0c, 0x00,
	0x18, 0x61,
	0x17, 0x11,
	0x32, 0x89,
	0x19, 0x02,
	0x1a, 0x7a,
	0x03, 0x00,
	0x0e, 0x84,
	0x0f, 0x62,
	0x15, 0x02,
	0x1b, 0x01,
	0x1e, 0x01,
	0x29, 0x3c,
	0x33, 0x00,
	0x34, 0x07,
	0x35, 0x84,
	0x36, 0x00,
	0x38, 0x13,
	0x39, 0x43,
	0x3c, 0x68,
	0x3d, 0x19,
	0x40, 0xc1,
	0x69, 0x80,
	0x6b, 0x0a,
	0xa1, 0x08,
	0x8b, 0xcc,
	0x8c, 0xcc,
	0x8d, 0xcf,
	0x14, 0x2e,
	0x25, 0x58,
	0x24, 0x68,
	0x2a, 0x10,
	OV7673_REGEND, 0x00	    /* End of list delimiter */
};

const static u8 gRGB565_CIF_15fps_INITIALIZE[] =
{
	0x12, 0x80,
	0x11, 0x03,
	0x3a, 0x04,
	0x12, 0x04,
	0x40, 0xd0,
	0x17, 0x15,
	0x18, 0x0b,
	0x32, 0xb6,
	0x19, 0x03,
	0x1a, 0x7b,
	0x03, 0x0a,
	0x0c, 0x08,
	0x3e, 0x11,
	0x70, 0x3a,
	0x71, 0x35,
	0x72, 0x11,
	0x73, 0xf1,
	0xa2, 0x02,
	0x7a, 0x20,
	0x7b, 0x1c,
	0x7c, 0x28,
	0x7d, 0x3c,
	0x7e, 0x5a,
	0x7f, 0x68,
	0x80, 0x76,
	0x81, 0x80,
	0x82, 0x88,
	0x83, 0x8f,
	0x84, 0x96,
	0x85, 0xa3,
	0x86, 0xaf,
	0x87, 0xc4,
	0x88, 0xd7,
	0x89, 0xe8,
	0x13, 0xe0,
	0x00, 0x00,
	0x10, 0x00,
	0x0d, 0x40,
	0x14, 0x38,
	0xa5, 0x05,
	0xab, 0x07,
	0x24, 0x95,
	0x25, 0x33,
	0x26, 0xe3,
	0x9f, 0x78,
	0xa0, 0x68,
	0xa1, 0x03,
	0xa6, 0xd8,
	0xa7, 0xd8,
	0xa8, 0xf0,
	0xa9, 0x90,
	0xaa, 0x94,
	0x13, 0xe5,
	0x0e, 0x61,
	0x0f, 0x4b,
	0x16, 0x02,
	0x1e, 0x07,
	0x21, 0x02,
	0x22, 0x91,
	0x29, 0x07,
	0x33, 0x0b,
	0x35, 0x0b,
	0x37, 0x1d,
	0x38, 0x71,
	0x39, 0x2a,
	0x3c, 0x78,
	0x4d, 0x40,
	0x4e, 0x20,
	0x69, 0x00,
	0x6b, 0x4a,
	0x74, 0x19,
	0x8d, 0x4f,
	0x8e, 0x00,
	0x8f, 0x00,
	0x90, 0x00,
	0x91, 0x00,
	0x96, 0x00,
	0x9a, 0x80,
	0xb0, 0x84,
	0xb1, 0x0c,
	0xb2, 0x0e,
	0xb3, 0x82,
	0xb8, 0x0a,
	0x43, 0x14,
	0x44, 0xf0,
	0x45, 0x34,
	0x46, 0x58,
	0x47, 0x28,
	0x48, 0x3a,
	0x59, 0x88,
	0x5a, 0x88,
	0x5b, 0x44,
	0x5c, 0x67,
	0x5d, 0x49,
	0x5e, 0x0e,
	0x6c, 0x0a,
	0x6d, 0x55,
	0x6e, 0x11,
	0x6f, 0x9f,
	0x6a, 0x40,
	0x01, 0x40,
	0x02, 0x40,
	0x13, 0xe7,
	0x4f, 0x80,
	0x50, 0x80,
	0x51, 0x00,
	0x52, 0x22,
	0x53, 0x5e,
	0x54, 0x80,
	0x58, 0x9e,
	0x41, 0x08,
	0x3f, 0x00,
	0x75, 0x05,
	0x76, 0xe1,
	0x4c, 0x00,
	0x77, 0x01,
	0x3d, 0xc0,
	0x4b, 0x09,
	0xc9, 0x60,
	0x41, 0x38,
	0x56, 0x40,
	0x34, 0x11,
	0x3b, 0x02,
	0xa4, 0x88,
	0x96, 0x00,
	0x97, 0x30,
	0x98, 0x20,
	0x99, 0x30,
	0x9a, 0x84,
	0x9b, 0x29,
	0x9c, 0x03,
	0x9d, 0x4c,
	0x9e, 0x3f,
	0x78, 0x04,
	0x79, 0x01,
	0xc8, 0xf0,
	0x79, 0x0f,
	0xc8, 0x00,
	0x79, 0x10,
	0xc8, 0x7e,
	0x79, 0x0a,
	0xc8, 0x80,
	0x79, 0x0b,
	0xc8, 0x01,
	0x79, 0x0c,
	0xc8, 0x0f,
	0x79, 0x0d,
	0xc8, 0x20,
	0x79, 0x09,
	0xc8, 0x80,
	0x79, 0x02,
	0xc8, 0xc0,
	0x79, 0x03,
	0xc8, 0x40,
	0x79, 0x05,
	0xc8, 0x30,
	0x79, 0x26,
	0xf1, 0x10,
	0x0f, 0x1d,
	0x0f, 0x1f,

	OV7673_REGEND, 0x00	    /* End of list delimiter */
};

const static u8 gRGB565_CIF_5fps_INITIALIZE[] =
{
	0x12, 0x80,
	0x11, 0x82,
	0x92, 0x48,
	0x93, 0x01,
	0x9d, 0x21,
	0x9e, 0x1b,
	0x3b, 0x02,
	0x13, 0xf2,
	0x10, 0x00,
	0x00, 0x00,
	0x01, 0x80,
	0x02, 0x80,
	0x13, 0xf7,
	0x12, 0x24,
	0x04, 0x00,
	0x18, 0x80,
	0x17, 0x28,
	0x32, 0x12,
	0x19, 0x02,
	0x1a, 0x7a,
	0x03, 0x00,
	0x0e, 0x84,
	0x0f, 0x62,
	0x15, 0x02,
	0x16, 0x02,
	0x1b, 0x01,
	0x1e, 0x01,
	0x29, 0x3c,
	0x33, 0x02,
	0x34, 0x00,
	0x35, 0x84,
	0x36, 0x03,
	0x38, 0x13,
	0x39, 0x43,
	0x3a, 0x00,
	0x3c, 0x6c,
	0x3d, 0x90,
	0x3f, 0x3c,
	0x40, 0xd1,
	0x41, 0x00,
	0x6b, 0x0f,
	0xa1, 0xc8,
	0x69, 0x80,
	0x43, 0xf0,
	0x44, 0x10,
	0x45, 0x78,
	0x46, 0xa8,
	0x47, 0x60,
	0x48, 0x80,
	0x59, 0xba,
	0x5a, 0x9a,
	0x5b, 0x22,
	0x5c, 0xb9,
	0x5d, 0x9b,
	0x5e, 0x10,
	0x5f, 0xe0,
	0x60, 0x85,
	0x61, 0x60,
	0x9f, 0x9d,
	0xa0, 0xe0,
	0x4f, 0x6c,
	0x50, 0x02,
	0x51, 0x12,
	0x52, 0x09,
	0x53, 0x6a,
	0x54, 0x0d,
	0x55, 0x12,
	0x56, 0x0d,
	0x57, 0x7b,
	0x58, 0x40,
	0x8b, 0xcc,
	0x8c, 0xcc,
	0x8d, 0xcf,
	0x6c, 0x40,
	0x6d, 0x30,
	0x6e, 0x4b,
	0x6f, 0x60,
	0x70, 0x70,
	0x71, 0x70,
	0x72, 0x70,
	0x73, 0x70,
	0x74, 0x60,
	0x75, 0x60,
	0x76, 0x50,
	0x77, 0x48,
	0x78, 0x3a,
	0x79, 0x2e,
	0x7a, 0x28,
	0x7b, 0x22,
	0x7c, 0x04,
	0x7d, 0x07,
	0x7e, 0x10,
	0x7f, 0x28,
	0x80, 0x36,
	0x81, 0x44,
	0x82, 0x52,
	0x83, 0x60,
	0x84, 0x6c,
	0x85, 0x78,
	0x86, 0x8c,
	0x87, 0x9e,
	0x88, 0xbb,
	0x89, 0xd2,
	0x8a, 0xe6,
	0x14, 0x4e,
	0x24, 0x68,
	0x25, 0x58,
	OV7673_REGEND, 0x00	    /* End of list delimiter */
};

const static u8 gRGB565_QCIF_15fps_INITIALIZE[] =
{
	0x12, 0x80,
	0x11, 0x03,
	0x3a, 0x04,
	0x12, 0x04,
	0x40, 0xd0,
	0x17, 0x39,
	0x18, 0x03,
	0x32, 0x80,
	0x19, 0x03,
	0x1a, 0x7b,
	0x03, 0x0a,
	0x0c, 0x0c,
	0x3e, 0x11,
	0x70, 0x3a,
	0x71, 0x35,
	0x72, 0x11,
	0x73, 0xf1,
	0xa2, 0x52,
	0x7a, 0x20,
	0x7b, 0x1c,
	0x7c, 0x28,
	0x7d, 0x3c,
	0x7e, 0x5a,
	0x7f, 0x68,
	0x80, 0x76,
	0x81, 0x80,
	0x82, 0x88,
	0x83, 0x8f,
	0x84, 0x96,
	0x85, 0xa3,
	0x86, 0xaf,
	0x87, 0xc4,
	0x88, 0xd7,
	0x89, 0xe8,
	0x13, 0xe0,
	0x00, 0x00,
	0x10, 0x00,
	0x0d, 0x40,
	0x14, 0x38,
	0xa5, 0x05,
	0xab, 0x07,
	0x24, 0x95,
	0x25, 0x33,
	0x26, 0xe3,
	0x9f, 0x78,
	0xa0, 0x68,
	0xa1, 0x03,
	0xa6, 0xd8,
	0xa7, 0xd8,
	0xa8, 0xf0,
	0xa9, 0x90,
	0xaa, 0x94,
	0x13, 0xe5,
	0x0e, 0x61,
	0x0f, 0x4b,
	0x16, 0x02,
	0x1e, 0x07,
	0x21, 0x02,
	0x22, 0x91,
	0x29, 0x07,
	0x33, 0x0b,
	0x35, 0x0b,
	0x37, 0x1d,
	0x38, 0x71,
	0x39, 0x2a,
	0x3c, 0x78,
	0x4d, 0x40,
	0x4e, 0x20,
	0x69, 0x00,
	0x6b, 0x4a,
	0x74, 0x19,
	0x8d, 0x4f,
	0x8e, 0x00,
	0x8f, 0x00,
	0x90, 0x00,
	0x91, 0x00,
	0x96, 0x00,
	0x9a, 0x80,
	0xb0, 0x84,
	0xb1, 0x0c,
	0xb2, 0x0e,
	0xb3, 0x82,
	0xb8, 0x0a,
	0x43, 0x14,
	0x44, 0xf0,
	0x45, 0x34,
	0x46, 0x58,
	0x47, 0x28,
	0x48, 0x3a,
	0x59, 0x88,
	0x5a, 0x88,
	0x5b, 0x44,
	0x5c, 0x67,
	0x5d, 0x49,
	0x5e, 0x0e,
	0x6c, 0x0a,
	0x6d, 0x55,
	0x6e, 0x11,
	0x6f, 0x9f,
	0x6a, 0x40,
	0x01, 0x40,
	0x02, 0x40,
	0x13, 0xe7,
	0x4f, 0x80,
	0x50, 0x80,
	0x51, 0x00,
	0x52, 0x22,
	0x53, 0x5e,
	0x54, 0x80,
	0x58, 0x9e,
	0x41, 0x08,
	0x3f, 0x00,
	0x75, 0x05,
	0x76, 0xe1,
	0x4c, 0x00,
	0x77, 0x01,
	0x3d, 0xc1,
	0x4b, 0x09,
	0xc9, 0x60,
	0x41, 0x38,
	0x56, 0x40,
	0x34, 0x11,
	0x3b, 0x02,
	0xa4, 0x88,
	0x96, 0x00,
	0x97, 0x30,
	0x98, 0x20,
	0x99, 0x30,
	0x9a, 0x84,
	0x9b, 0x29,
	0x9c, 0x03,
	0x9d, 0x4c,
	0x9e, 0x3f,
	0x78, 0x04,
	0x79, 0x01,
	0xc8, 0xf0,
	0x79, 0x0f,
	0xc8, 0x00,
	0x79, 0x10,
	0xc8, 0x7e,
	0x79, 0x0a,
	0xc8, 0x80,
	0x79, 0x0b,
	0xc8, 0x01,
	0x79, 0x0c,
	0xc8, 0x0f,
	0x79, 0x0d,
	0xc8, 0x20,
	0x79, 0x09,
	0xc8, 0x80,
	0x79, 0x02,
	0xc8, 0xc0,
	0x79, 0x03,
	0xc8, 0x40,
	0x79, 0x05,
	0xc8, 0x30,
	0x79, 0x26,
	0xf1, 0x10,
	0x0f, 0x1d,
	0x0f, 0x1f,
	OV7673_REGEND, 0x00	    /* End of list delimiter */
};

const static u8 gRGB565_QCIF_5fps_INITIALIZE[] =
{
	0x12, 0x80,
	0x11, 0x80,
	0x92, 0x68,
	0x93, 0x06,
	0x9d, 0x62,
	0x9e, 0x52,
	0x3b, 0x02,
	0x13, 0xf2,
	0x10, 0x00,
	0x00, 0x00,
	0x01, 0x80,
	0x02, 0x80,
	0x13, 0xf7,
	0x12, 0x0c,
	0x04, 0x04,
	0x18, 0x80,
	0x17, 0x28,
	0x32, 0x92,
	0x19, 0x02,
	0x1a, 0x4a,
	0x03, 0x00,
	0x0e, 0x84,
	0x0f, 0x62,
	0x15, 0x02,
	0x16, 0x02,
	0x1b, 0x01,
	0x1e, 0x01,
	0x29, 0x3c,
	0x33, 0x02,
	0x34, 0x00,
	0x35, 0x84,
	0x36, 0x03,
	0x38, 0x13,
	0x39, 0x43,
	0x3a, 0x00,
	0x3c, 0x6c,
	0x3d, 0x90,
	0x3f, 0x3c,
	0x40, 0xd1,
	0x41, 0x00,
	0x6b, 0x0f,
	0xa1, 0xc8,
	0x69, 0x80,
	0x43, 0xf0,
	0x44, 0x10,
	0x45, 0x78,
	0x46, 0xa8,
	0x47, 0x60,
	0x48, 0x80,
	0x59, 0xba,
	0x5a, 0x9a,
	0x5b, 0x22,
	0x5c, 0xb9,
	0x5d, 0x9b,
	0x5e, 0x10,
	0x5f, 0xe0,
	0x60, 0x85,
	0x61, 0x60,
	0x9f, 0x9d,
	0xa0, 0xe0,
	0x4f, 0x6c,
	0x50, 0x02,
	0x51, 0x12,
	0x52, 0x09,
	0x53, 0x6a,
	0x54, 0x0d,
	0x55, 0x12,
	0x56, 0x0d,
	0x57, 0x7b,
	0x58, 0x40,
	0x8b, 0xcc,
	0x8c, 0xcc,
	0x8d, 0xcf,
	0x6c, 0x40,
	0x6d, 0x30,
	0x6e, 0x4b,
	0x6f, 0x60,
	0x70, 0x70,
	0x71, 0x70,
	0x72, 0x70,
	0x73, 0x70,
	0x74, 0x60,
	0x75, 0x60,
	0x76, 0x50,
	0x77, 0x48,
	0x78, 0x3a,
	0x79, 0x2e,
	0x7a, 0x28,
	0x7b, 0x22,
	0x7c, 0x04,
	0x7d, 0x07,
	0x7e, 0x10,
	0x7f, 0x28,
	0x80, 0x36,
	0x81, 0x44,
	0x82, 0x52,
	0x83, 0x60,
	0x84, 0x6c,
	0x85, 0x78,
	0x86, 0x8c,
	0x87, 0x9e,
	0x88, 0xbb,
	0x89, 0xd2,
	0x8a, 0xe6,
	0x14, 0x4e,
	0x24, 0x68,
	0x25, 0x58,
	OV7673_REGEND, 0x00	    /* End of list delimiter */
};

const static u8 gRGB565_QQCIF_15fps_INITIALIZE[] =
{
	0x12, 0x80,
	0x11, 0x03,
	0x3a, 0x04,
	0x12, 0x04,
	0x40, 0xd0,
	0x17, 0x3a,
	0x18, 0x04,
	0x32, 0xb6,
	0x19, 0x03,
	0x1a, 0x7b,
	0x03, 0x0a,
	0x0c, 0x0c,
	0x3e, 0x12,
	0x70, 0x3a,
	0x71, 0x35,
	0x72, 0x22,
	0x73, 0xf2,
	0xa2, 0x2a,
	0x7a, 0x20,
	0x7b, 0x1c,
	0x7c, 0x28,
	0x7d, 0x3c,
	0x7e, 0x5a,
	0x7f, 0x68,
	0x80, 0x76,
	0x81, 0x80,
	0x82, 0x88,
	0x83, 0x8f,
	0x84, 0x96,
	0x85, 0xa3,
	0x86, 0xaf,
	0x87, 0xc4,
	0x88, 0xd7,
	0x89, 0xe8,
	0x13, 0xe0,
	0x00, 0x00,
	0x10, 0x00,
	0x0d, 0x40,
	0x14, 0x38,
	0xa5, 0x05,
	0xab, 0x07,
	0x24, 0x95,
	0x25, 0x33,
	0x26, 0xe3,
	0x9f, 0x78,
	0xa0, 0x68,
	0xa1, 0x03,
	0xa6, 0xd8,
	0xa7, 0xd8,
	0xa8, 0xf0,
	0xa9, 0x90,
	0xaa, 0x94,
	0x13, 0xe5,
	0x0e, 0x61,
	0x0f, 0x4b,
	0x16, 0x02,
	0x1e, 0x07,
	0x21, 0x02,
	0x22, 0x91,
	0x29, 0x07,
	0x33, 0x0b,
	0x35, 0x0b,
	0x37, 0x1d,
	0x38, 0x71,
	0x39, 0x2a,
	0x3c, 0x78,
	0x4d, 0x40,
	0x4e, 0x20,
	0x69, 0x00,
	0x6b, 0x4a,
	0x74, 0x19,
	0x8d, 0x4f,
	0x8e, 0x00,
	0x8f, 0x00,
	0x90, 0x00,
	0x91, 0x00,
	0x96, 0x00,
	0x9a, 0x80,
	0xb0, 0x84,
	0xb1, 0x0c,
	0xb2, 0x0e,
	0xb3, 0x82,
	0xb8, 0x0a,
	0x43, 0x14,
	0x44, 0xf0,
	0x45, 0x34,
	0x46, 0x58,
	0x47, 0x28,
	0x48, 0x3a,
	0x59, 0x88,
	0x5a, 0x88,
	0x5b, 0x44,
	0x5c, 0x67,
	0x5d, 0x49,
	0x5e, 0x0e,
	0x6c, 0x0a,
	0x6d, 0x55,
	0x6e, 0x11,
	0x6f, 0x9f,
	0x6a, 0x40,
	0x01, 0x40,
	0x02, 0x40,
	0x13, 0xe7,
	0x4f, 0x80,
	0x50, 0x80,
	0x51, 0x00,
	0x52, 0x22,
	0x53, 0x5e,
	0x54, 0x80,
	0x58, 0x9e,
	0x41, 0x08,
	0x3f, 0x00,
	0x75, 0x05,
	0x76, 0xe1,
	0x4c, 0x00,
	0x77, 0x01,
	0x3d, 0xc0,
	0x4b, 0x09,
	0xc9, 0x60,
	0x41, 0x38,
	0x56, 0x40,
	0x34, 0x11,
	0x3b, 0x02,
	0xa4, 0x88,
	0x96, 0x00,
	0x97, 0x30,
	0x98, 0x20,
	0x99, 0x30,
	0x9a, 0x84,
	0x9b, 0x29,
	0x9c, 0x03,
	0x9d, 0x4c,
	0x9e, 0x3f,
	0x78, 0x04,
	0x79, 0x01,
	0xc8, 0xf0,
	0x79, 0x0f,
	0xc8, 0x00,
	0x79, 0x10,
	0xc8, 0x7e,
	0x79, 0x0a,
	0xc8, 0x80,
	0x79, 0x0b,
	0xc8, 0x01,
	0x79, 0x0c,
	0xc8, 0x0f,
	0x79, 0x0d,
	0xc8, 0x20,
	0x79, 0x09,
	0xc8, 0x80,
	0x79, 0x02,
	0xc8, 0xc0,
	0x79, 0x03,
	0xc8, 0x40,
	0x79, 0x05,
	0xc8, 0x30,
	0x79, 0x26,
	0xf1, 0x10,
	0x0f, 0x1d,
	0x0f, 0x1f,

	OV7673_REGEND, 0x00	    /* End of list delimiter */
};

const static u8 gRGB565_QQCIF_5fps_INITIALIZE[] =
{
	0x12, 0x80,
	0x11, 0x80,
	0x92, 0x68,
	0x93, 0x06,
	0x9d, 0x62,
	0x9e, 0x52,
	0x3b, 0x02,
	0x13, 0xf2,
	0x10, 0x00,
	0x00, 0x00,
	0x01, 0x80,
	0x02, 0x80,
	0x13, 0xf7,
	0x12, 0x0c,
	0x04, 0x24,
	0x18, 0x80,
	0x17, 0x28,
	0x32, 0x92,
	0x19, 0x02,
	0x1a, 0x4a,
	0x03, 0x00,
	0x0e, 0x84,
	0x0f, 0x62,
	0x15, 0x02,
	0x16, 0x02,
	0x1b, 0x01,
	0x1e, 0x01,
	0x29, 0x3c,
	0x33, 0x02,
	0x34, 0x00,
	0x35, 0x84,
	0x36, 0x03,
	0x38, 0x13,
	0x39, 0x43,
	0x3a, 0x00,
	0x3c, 0x6c,
	0x3d, 0x90,
	0x3f, 0x3c,
	0x40, 0xd1,
	0x41, 0x00,
	0x6b, 0x0f,
	0xa1, 0xc8,
	0x69, 0x80,
	0x43, 0xf0,
	0x44, 0x10,
	0x45, 0x78,
	0x46, 0xa8,
	0x47, 0x60,
	0x48, 0x80,
	0x59, 0xba,
	0x5a, 0x9a,
	0x5b, 0x22,
	0x5c, 0xb9,
	0x5d, 0x9b,
	0x5e, 0x10,
	0x5f, 0xe0,
	0x60, 0x85,
	0x61, 0x60,
	0x9f, 0x9d,
	0xa0, 0xe0,
	0x4f, 0x6c,
	0x50, 0x02,
	0x51, 0x12,
	0x52, 0x09,
	0x53, 0x6a,
	0x54, 0x0d,
	0x55, 0x12,
	0x56, 0x0d,
	0x57, 0x7b,
	0x58, 0x40,
	0x8b, 0xcc,
	0x8c, 0xcc,
	0x8d, 0xcf,
	0x6c, 0x40,
	0x6d, 0x30,
	0x6e, 0x4b,
	0x6f, 0x60,
	0x70, 0x70,
	0x71, 0x70,
	0x72, 0x70,
	0x73, 0x70,
	0x74, 0x60,
	0x75, 0x60,
	0x76, 0x50,
	0x77, 0x48,
	0x78, 0x3a,
	0x79, 0x2e,
	0x7a, 0x28,
	0x7b, 0x22,
	0x7c, 0x04,
	0x7d, 0x07,
	0x7e, 0x10,
	0x7f, 0x28,
	0x80, 0x36,
	0x81, 0x44,
	0x82, 0x52,
	0x83, 0x60,
	0x84, 0x6c,
	0x85, 0x78,
	0x86, 0x8c,
	0x87, 0x9e,
	0x88, 0xbb,
	0x89, 0xd2,
	0x8a, 0xe6,
	0x14, 0x4e,
	0x24, 0x68,
	0x25, 0x58,
	OV7673_REGEND, 0x00	    /* End of list delimiter */
};

const static u8 gRGB565_QQVGA_15fps_INITIALIZE[] =
{
	0x12, 0x80,
	0x11, 0x03,
	0x3a, 0x04,
	0x12, 0x04,
	0x40, 0xd0,
	0x32, 0x24,
	0x17, 0x16,
	0x18, 0x04,
	0x19, 0x02,
	0x1a, 0x7a,
	0x03, 0x0a,
	0x0c, 0x04,
	0x3e, 0x1a,
	0x70, 0x3a,
	0x71, 0x35,
	0x72, 0x22,
	0x73, 0xf2,
	0xa2, 0x02,
	0x7a, 0x20,
	0x7b, 0x1c,
	0x7c, 0x28,
	0x7d, 0x3c,
	0x7e, 0x5a,
	0x7f, 0x68,
	0x80, 0x76,
	0x81, 0x80,
	0x82, 0x88,
	0x83, 0x8f,
	0x84, 0x96,
	0x85, 0xa3,
	0x86, 0xaf,
	0x87, 0xc4,
	0x88, 0xd7,
	0x89, 0xe8,
	0x13, 0xe0,
	0x00, 0x00,
	0x10, 0x00,
	0x0d, 0x40,
	0x14, 0x38,
	0xa5, 0x05,
	0xab, 0x07,
	0x24, 0x95,
	0x25, 0x33,
	0x26, 0xe3,
	0x9f, 0x78,
	0xa0, 0x68,
	0xa1, 0x03,
	0xa6, 0xd8,
	0xa7, 0xd8,
	0xa8, 0xf0,
	0xa9, 0x90,
	0xaa, 0x94,
	0x13, 0xe5,
	0x0e, 0x61,
	0x0f, 0x4b,
	0x16, 0x02,
	0x1e, 0x07,
	0x21, 0x02,
	0x22, 0x91,
	0x29, 0x07,
	0x33, 0x0b,
	0x35, 0x0b,
	0x37, 0x1d,
	0x38, 0x71,
	0x39, 0x2a,
	0x3c, 0x78,
	0x4d, 0x40,
	0x4e, 0x20,
	0x69, 0x00,
	0x6b, 0x4a,
	0x74, 0x19,
	0x8d, 0x4f,
	0x8e, 0x00,
	0x8f, 0x00,
	0x90, 0x00,
	0x91, 0x00,
	0x96, 0x00,
	0x9a, 0x80,
	0xb0, 0x84,
	0xb1, 0x0c,
	0xb2, 0x0e,
	0xb3, 0x82,
	0xb8, 0x0a,
	0x43, 0x14,
	0x44, 0xf0,
	0x45, 0x34,
	0x46, 0x58,
	0x47, 0x28,
	0x48, 0x3a,
	0x59, 0x88,
	0x5a, 0x88,
	0x5b, 0x44,
	0x5c, 0x67,
	0x5d, 0x49,
	0x5e, 0x0e,
	0x6c, 0x0a,
	0x6d, 0x55,
	0x6e, 0x11,
	0x6f, 0x9f,
	0x6a, 0x40,
	0x01, 0x40,
	0x02, 0x40,
	0x13, 0xe7,
	0x4f, 0x80,
	0x50, 0x80,
	0x51, 0x00,
	0x52, 0x22,
	0x53, 0x5e,
	0x54, 0x80,
	0x58, 0x9e,
	0x41, 0x08,
	0x3f, 0x00,
	0x75, 0x05,
	0x76, 0xe1,
	0x4c, 0x00,
	0x77, 0x01,
	0x3d, 0xc1,
	0x4b, 0x09,
	0xc9, 0x60,
	0x41, 0x38,
	0x56, 0x40,
	0x34, 0x11,
	0x3b, 0x02,
	0xa4, 0x88,
	0x96, 0x00,
	0x97, 0x30,
	0x98, 0x20,
	0x99, 0x30,
	0x9a, 0x84,
	0x9b, 0x29,
	0x9c, 0x03,
	0x9d, 0x4c,
	0x9e, 0x3f,
	0x78, 0x04,
	0x79, 0x01,
	0xc8, 0xf0,
	0x79, 0x0f,
	0xc8, 0x00,
	0x79, 0x10,
	0xc8, 0x7e,
	0x79, 0x0a,
	0xc8, 0x80,
	0x79, 0x0b,
	0xc8, 0x01,
	0x79, 0x0c,
	0xc8, 0x0f,
	0x79, 0x0d,
	0xc8, 0x20,
	0x79, 0x09,
	0xc8, 0x80,
	0x79, 0x02,
	0xc8, 0xc0,
	0x79, 0x03,
	0xc8, 0x40,
	0x79, 0x05,
	0xc8, 0x30,
	0x79, 0x26,
	0xf1, 0x10,
	0x0f, 0x1d,
	0x0f, 0x1f,

	OV7673_REGEND, 0x00	    /* End of list delimiter */
};

const static u8 gRGB565_QQVGA_5fps_INITIALIZE[] =
{
	0x12, 0x80,
	0x11, 0x82,
	0x92, 0xd8,
	0x93, 0x03,
	0x9d, 0x41,
	0x9e, 0x36,
	0x3b, 0x02,
	0x13, 0xf2,
	0x10, 0x00,
	0x00, 0x00,
	0x01, 0x80,
	0x02, 0x80,
	0x13, 0xf7,
	0x12, 0x14,
	0x04, 0x24,
	0x18, 0x4b,
	0x17, 0x23,
	0x32, 0x3f,
	0x19, 0x02,
	0x1a, 0x7a,
	0x03, 0x00,
	0x0e, 0x84,
	0x0f, 0x62,
	0x15, 0x02,
	0x16, 0x02,
	0x1b, 0x01,
	0x1e, 0x01,
	0x29, 0x3c,
	0x33, 0x00,
	0x34, 0x07,
	0x35, 0x84,
	0x36, 0x00,
	0x38, 0x13,
	0x39, 0x43,
	0x3a, 0x00,
	0x3c, 0x6c,
	0x3d, 0x90,
	0x3f, 0x3c,
	0x40, 0xd1,
	0x41, 0x00,
	0x6b, 0x0a,
	0xa1, 0xc8,
	0x69, 0x80,
	0x43, 0xf0,
	0x44, 0x10,
	0x45, 0x78,
	0x46, 0xa8,
	0x47, 0x60,
	0x48, 0x80,
	0x59, 0xba,
	0x5a, 0x9a,
	0x5b, 0x22,
	0x5c, 0xb9,
	0x5d, 0x9b,
	0x5e, 0x10,
	0x5f, 0xe0,
	0x60, 0x85,
	0x61, 0x60,
	0x9f, 0x9d,
	0xa0, 0xe0,
	0x4f, 0x6c,
	0x50, 0x02,
	0x51, 0x12,
	0x52, 0x09,
	0x53, 0x6a,
	0x54, 0x0d,
	0x55, 0x12,
	0x56, 0x0d,
	0x57, 0x7b,
	0x58, 0x40,
	0x8b, 0xcc,
	0x8c, 0xcc,
	0x8d, 0xcf,
	0x6c, 0x40,
	0x6d, 0x30,
	0x6e, 0x4b,
	0x6f, 0x60,
	0x70, 0x70,
	0x71, 0x70,
	0x72, 0x70,
	0x73, 0x70,
	0x74, 0x60,
	0x75, 0x60,
	0x76, 0x50,
	0x77, 0x48,
	0x78, 0x3a,
	0x79, 0x2e,
	0x7a, 0x28,
	0x7b, 0x22,
	0x7c, 0x04,
	0x7d, 0x07,
	0x7e, 0x10,
	0x7f, 0x28,
	0x80, 0x36,
	0x81, 0x44,
	0x82, 0x52,
	0x83, 0x60,
	0x84, 0x6c,
	0x85, 0x78,
	0x86, 0x8c,
	0x87, 0x9e,
	0x88, 0xbb,
	0x89, 0xd2,
	0x8a, 0xe6,
	0x14, 0x4e,
	0x24, 0x68,
	0x25, 0x58,
	OV7673_REGEND, 0x00	    /* End of list delimiter */
};

const static u8 gRGB565_QVGA_15fps_INITIALIZE[] =
{
	0x12, 0x80,
	0x11, 0x03,
	0x3a, 0x04,
	0x12, 0x04,
	0x40, 0xd0,
	0x32, 0x80,
	0x17, 0x16,
	0x18, 0x04,
	0x19, 0x02,
	0x1a, 0x7a,
	0x03, 0x0a,
	0x0c, 0x04,
	0x3e, 0x19,
	0x70, 0x3a,
	0x71, 0x35,
	0x72, 0x11,
	0x73, 0xf1,
	0xa2, 0x02,
	0x7a, 0x20,
	0x7b, 0x1c,
	0x7c, 0x28,
	0x7d, 0x3c,
	0x7e, 0x5a,
	0x7f, 0x68,
	0x80, 0x76,
	0x81, 0x80,
	0x82, 0x88,
	0x83, 0x8f,
	0x84, 0x96,
	0x85, 0xa3,
	0x86, 0xaf,
	0x87, 0xc4,
	0x88, 0xd7,
	0x89, 0xe8,
	0x13, 0xe0,
	0x00, 0x00,
	0x10, 0x00,
	0x0d, 0x40,
	0x14, 0x38,
	0xa5, 0x05,
	0xab, 0x07,
	0x24, 0x95,
	0x25, 0x33,
	0x26, 0xe3,
	0x9f, 0x78,
	0xa0, 0x68,
	0xa1, 0x03,
	0xa6, 0xd8,
	0xa7, 0xd8,
	0xa8, 0xf0,
	0xa9, 0x90,
	0xaa, 0x94,
	0x13, 0xe5,
	0x0e, 0x61,
	0x0f, 0x4b,
	0x16, 0x02,
	0x1e, 0x07,
	0x21, 0x02,
	0x22, 0x91,
	0x29, 0x07,
	0x33, 0x0b,
	0x35, 0x0b,
	0x37, 0x1d,
	0x38, 0x71,
	0x39, 0x2a,
	0x3c, 0x78,
	0x4d, 0x40,
	0x4e, 0x20,
	0x69, 0x00,
	0x6b, 0x4a,
	0x74, 0x19,
	0x8d, 0x4f,
	0x8e, 0x00,
	0x8f, 0x00,
	0x90, 0x00,
	0x91, 0x00,
	0x96, 0x00,
	0x9a, 0x80,
	0xb0, 0x84,
	0xb1, 0x0c,
	0xb2, 0x0e,
	0xb3, 0x82,
	0xb8, 0x0a,
	0x43, 0x14,
	0x44, 0xf0,
	0x45, 0x34,
	0x46, 0x58,
	0x47, 0x28,
	0x48, 0x3a,
	0x59, 0x88,
	0x5a, 0x88,
	0x5b, 0x44,
	0x5c, 0x67,
	0x5d, 0x49,
	0x5e, 0x0e,
	0x6c, 0x0a,
	0x6d, 0x55,
	0x6e, 0x11,
	0x6f, 0x9f,
	0x6a, 0x40,
	0x01, 0x40,
	0x02, 0x40,
	0x13, 0xe7,
	0x4f, 0x80,
	0x50, 0x80,
	0x51, 0x00,
	0x52, 0x22,
	0x53, 0x5e,
	0x54, 0x80,
	0x58, 0x9e,
	0x41, 0x08,
	0x3f, 0x00,
	0x75, 0x05,
	0x76, 0xe1,
	0x4c, 0x00,
	0x77, 0x01,
	0x3d, 0xc0,
	0x4b, 0x09,
	0xc9, 0x60,
	0x41, 0x38,
	0x56, 0x40,
	0x34, 0x11,
	0x3b, 0x02,
	0xa4, 0x88,
	0x96, 0x00,
	0x97, 0x30,
	0x98, 0x20,
	0x99, 0x30,
	0x9a, 0x84,
	0x9b, 0x29,
	0x9c, 0x03,
	0x9d, 0x4c,
	0x9e, 0x3f,
	0x78, 0x04,
	0x79, 0x01,
	0xc8, 0xf0,
	0x79, 0x0f,
	0xc8, 0x00,
	0x79, 0x10,
	0xc8, 0x7e,
	0x79, 0x0a,
	0xc8, 0x80,
	0x79, 0x0b,
	0xc8, 0x01,
	0x79, 0x0c,
	0xc8, 0x0f,
	0x79, 0x0d,
	0xc8, 0x20,
	0x79, 0x09,
	0xc8, 0x80,
	0x79, 0x02,
	0xc8, 0xc0,
	0x79, 0x03,
	0xc8, 0x40,
	0x79, 0x05,
	0xc8, 0x30,
	0x79, 0x26,
	0xf1, 0x10,
	0x0f, 0x1d,
	0x0f, 0x1f,

	OV7673_REGEND, 0x00	    /* End of list delimiter */
};

const static u8 gRGB565_QVGA_5fps_INITIALIZE[] =
{
	0x12, 0x80,
	0x11, 0x85,
	0x92, 0x48,
	0x93, 0x01,
	0x9d, 0x21,
	0x9e, 0x1b,
	0x3b, 0x02,
	0x13, 0xf2,
	0x10, 0x00,
	0x00, 0x00,
	0x01, 0x80,
	0x02, 0x80,
	0x13, 0xf7,
	0x12, 0x14,
	0x04, 0x00,
	0x18, 0x4b,
	0x17, 0x23,
	0x32, 0xbf,
	0x19, 0x02,
	0x1a, 0x7a,
	0x03, 0x00,
	0x0e, 0x84,
	0x0f, 0x62,
	0x15, 0x02,
	0x16, 0x02,
	0x1b, 0x01,
	0x1e, 0x01,
	0x29, 0x3c,
	0x33, 0x00,
	0x34, 0x07,
	0x35, 0x84,
	0x36, 0x00,
	0x38, 0x13,
	0x39, 0x43,
	0x3a, 0x00,
	0x3c, 0x6c,
	0x3d, 0x90,
	0x3f, 0x3c,
	0x40, 0xd1,
	0x41, 0x00,
	0x6b, 0x0a,
	0xa1, 0xc8,
	0x69, 0x80,
	0x43, 0xf0,
	0x44, 0x10,
	0x45, 0x78,
	0x46, 0xa8,
	0x47, 0x60,
	0x48, 0x80,
	0x59, 0xba,
	0x5a, 0x9a,
	0x5b, 0x22,
	0x5c, 0xb9,
	0x5d, 0x9b,
	0x5e, 0x10,
	0x5f, 0xe0,
	0x60, 0x85,
	0x61, 0x60,
	0x9f, 0x9d,
	0xa0, 0xe0,
	0x4f, 0x6c,
	0x50, 0x02,
	0x51, 0x12,
	0x52, 0x09,
	0x53, 0x6a,
	0x54, 0x0d,
	0x55, 0x12,
	0x56, 0x0d,
	0x57, 0x7b,
	0x58, 0x40,
	0x8b, 0xcc,
	0x8c, 0xcc,
	0x8d, 0xcf,
	0x6c, 0x40,
	0x6d, 0x30,
	0x6e, 0x4b,
	0x6f, 0x60,
	0x70, 0x70,
	0x71, 0x70,
	0x72, 0x70,
	0x73, 0x70,
	0x74, 0x60,
	0x75, 0x60,
	0x76, 0x50,
	0x77, 0x48,
	0x78, 0x3a,
	0x79, 0x2e,
	0x7a, 0x28,
	0x7b, 0x22,
	0x7c, 0x04,
	0x7d, 0x07,
	0x7e, 0x10,
	0x7f, 0x28,
	0x80, 0x36,
	0x81, 0x44,
	0x82, 0x52,
	0x83, 0x60,
	0x84, 0x6c,
	0x85, 0x78,
	0x86, 0x8c,
	0x87, 0x9e,
	0x88, 0xbb,
	0x89, 0xd2,
	0x8a, 0xe6,
	0x14, 0x4e,
	0x24, 0x68,
	0x25, 0x58,
	OV7673_REGEND, 0x00	    /* End of list delimiter */
};

const static u8 gRGB565_VGA_15fps_INITIALIZE[] =
{
	0x12, 0x80,
	0x11, 0x03,
	0x3a, 0x04,
	0x12, 0x04,
	0x40, 0xd0,
	0x17, 0x13,
	0x18, 0x01,
	0x32, 0xb6,
	0x19, 0x02,
	0x1a, 0x7a,
	0x03, 0x0a,
	0x0c, 0x00,
	0x3e, 0x00,
	0x70, 0x3a,
	0x71, 0x35,
	0x72, 0x11,
	0x73, 0xf0,
	0xa2, 0x02,
	0x7a, 0x20,
	0x7b, 0x1c,
	0x7c, 0x28,
	0x7d, 0x3c,
	0x7e, 0x5a,
	0x7f, 0x68,
	0x80, 0x76,
	0x81, 0x80,
	0x82, 0x88,
	0x83, 0x8f,
	0x84, 0x96,
	0x85, 0xa3,
	0x86, 0xaf,
	0x87, 0xc4,
	0x88, 0xd7,
	0x89, 0xe8,
	0x13, 0xe0,
	0x00, 0x00,
	0x10, 0x00,
	0x0d, 0x40,
	0x14, 0x38,
	0xa5, 0x05,
	0xab, 0x07,
	0x24, 0x95,
	0x25, 0x33,
	0x26, 0xe3,
	0x9f, 0x78,
	0xa0, 0x68,
	0xa1, 0x03,
	0xa6, 0xd8,
	0xa7, 0xd8,
	0xa8, 0xf0,
	0xa9, 0x90,
	0xaa, 0x94,
	0x13, 0xe5,
	0x0e, 0x61,
	0x0f, 0x4b,
	0x16, 0x02,
	0x1e, 0x07,
	0x21, 0x02,
	0x22, 0x91,
	0x29, 0x07,
	0x33, 0x0b,
	0x35, 0x0b,
	0x37, 0x1d,
	0x38, 0x71,
	0x39, 0x2a,
	0x3c, 0x78,
	0x4d, 0x40,
	0x4e, 0x20,
	0x69, 0x00,
	0x6b, 0x4a,
	0x74, 0x19,
	0x8d, 0x4f,
	0x8e, 0x00,
	0x8f, 0x00,
	0x90, 0x00,
	0x91, 0x00,
	0x96, 0x00,
	0x9a, 0x80,
	0xb0, 0x84,
	0xb1, 0x0c,
	0xb2, 0x0e,
	0xb3, 0x82,
	0xb8, 0x0a,
	0x43, 0x14,
	0x44, 0xf0,
	0x45, 0x34,
	0x46, 0x58,
	0x47, 0x28,
	0x48, 0x3a,
	0x59, 0x88,
	0x5a, 0x88,
	0x5b, 0x44,
	0x5c, 0x67,
	0x5d, 0x49,
	0x5e, 0x0e,
	0x6c, 0x0a,
	0x6d, 0x55,
	0x6e, 0x11,
	0x6f, 0x9f,
	0x6a, 0x40,
	0x01, 0x40,
	0x02, 0x40,
	0x13, 0xe7,
	0x4f, 0x80,
	0x50, 0x80,
	0x51, 0x00,
	0x52, 0x22,
	0x53, 0x5e,
	0x54, 0x80,
	0x58, 0x9e,
	0x41, 0x08,
	0x3f, 0x00,
	0x75, 0x05,
	0x76, 0xe1,
	0x4c, 0x00,
	0x77, 0x01,
	0x3d, 0xc0,
	0x4b, 0x09,
	0xc9, 0x60,
	0x41, 0x38,
	0x56, 0x40,
	0x34, 0x11,
	0x3b, 0x02,
	0xa4, 0x88,
	0x96, 0x00,
	0x97, 0x30,
	0x98, 0x20,
	0x99, 0x30,
	0x9a, 0x84,
	0x9b, 0x29,
	0x9c, 0x03,
	0x9d, 0x4c,
	0x9e, 0x3f,
	0x78, 0x04,
	0x79, 0x01,
	0xc8, 0xf0,
	0x79, 0x0f,
	0xc8, 0x00,
	0x79, 0x10,
	0xc8, 0x7e,
	0x79, 0x0a,
	0xc8, 0x80,
	0x79, 0x0b,
	0xc8, 0x01,
	0x79, 0x0c,
	0xc8, 0x0f,
	0x79, 0x0d,
	0xc8, 0x20,
	0x79, 0x09,
	0xc8, 0x80,
	0x79, 0x02,
	0xc8, 0xc0,
	0x79, 0x03,
	0xc8, 0x40,
	0x79, 0x05,
	0xc8, 0x30,
	0x79, 0x26,
	0xf1, 0x10,
	0x0f, 0x1d,
	0x0f, 0x1f,

	OV7673_REGEND, 0x00	    /* End of list delimiter */
};

const static u8 gRGB565_VGA_5fps_INITIALIZE[] =
{
	0x12, 0x80,
	0x11, 0x85,
	0x92, 0x48,
	0x93, 0x01,
	0x9d, 0x19,
	0x9e, 0x15,
	0x3b, 0x02,
	0x13, 0xf2,
	0x10, 0x00,
	0x00, 0x00,
	0x01, 0x80,
	0x02, 0x80,
	0x13, 0xf7,
	0x12, 0x04,
	0x04, 0x00,
	0x18, 0x00,
	0x17, 0x12,
	0x32, 0xa4,
	0x19, 0x02,
	0x1a, 0x7a,
	0x03, 0x00,
	0x0e, 0x84,
	0x0f, 0x62,
	0x15, 0x02,
	0x16, 0x02,
	0x1b, 0x01,
	0x1e, 0x01,
	0x29, 0x3c,
	0x33, 0x00,
	0x34, 0x07,
	0x35, 0x84,
	0x36, 0x00,
	0x38, 0x13,
	0x39, 0x43,
	0x3a, 0x00,
	0x3c, 0x6c,
	0x3d, 0x90,
	0x3f, 0x3c,
	0x40, 0xd1,
	0x41, 0x00,
	0x6b, 0x0a,
	0xa1, 0xc8,
	0x69, 0x80,
	0x43, 0xf0,
	0x44, 0x10,
	0x45, 0x78,
	0x46, 0xa8,
	0x47, 0x60,
	0x48, 0x80,
	0x59, 0xba,
	0x5a, 0x9a,
	0x5b, 0x22,
	0x5c, 0xb9,
	0x5d, 0x9b,
	0x5e, 0x10,
	0x5f, 0xe0,
	0x60, 0x85,
	0x61, 0x60,
	0x9f, 0x9d,
	0xa0, 0xe0,
	0x4f, 0x6c,
	0x50, 0x02,
	0x51, 0x12,
	0x52, 0x09,
	0x53, 0x6a,
	0x54, 0x0d,
	0x55, 0x12,
	0x56, 0x0d,
	0x57, 0x7b,
	0x58, 0x40,
	0x8b, 0xcc,
	0x8c, 0xcc,
	0x8d, 0xcf,
	0x6c, 0xf0,
	0x6d, 0xf0,
	0x6e, 0xf0,
	0x6f, 0x70,
	0x70, 0x58,
	0x71, 0x58,
	0x72, 0x48,
	0x73, 0x48,
	0x74, 0x38,
	0x75, 0x40,
	0x76, 0x34,
	0x77, 0x34,
	0x78, 0x2e,
	0x79, 0x28,
	0x7a, 0x24,
	0x7b, 0x22,
	0x7c, 0x0f,
	0x7d, 0x1e,
	0x7e, 0x3c,
	0x7f, 0x58,
	0x80, 0x63,
	0x81, 0x6e,
	0x82, 0x77,
	0x83, 0x80,
	0x84, 0x87,
	0x85, 0x8f,
	0x86, 0x9c,
	0x87, 0xa9,
	0x88, 0xc0,
	0x89, 0xd4,
	0x8a, 0xe6,
	0x14, 0x4e,
	0x24, 0x68,
	0x25, 0x58,
	OV7673_REGEND, 0x00	    /* End of list delimiter */
};

const static u8 gYUV_CIF_15fps_INITIALIZE[] =
{
	0x12, 0x80,
	0x11, 0x03,
	0x3a, 0x08,
	0x12, 0x00,
	0x17, 0x15,
	0x18, 0x0b,
	0x32, 0xb6,
	0x19, 0x03,
	0x1a, 0x7b,
	0x03, 0x0a,
	0x0c, 0x08,
	0x3e, 0x11,
	0x70, 0x3a,
	0x71, 0x35,
	0x72, 0x11,
	0x73, 0xf1,
	0xa2, 0x02,
	0x7a, 0x20,
	0x7b, 0x1c,
	0x7c, 0x28,
	0x7d, 0x3c,
	0x7e, 0x5a,
	0x7f, 0x68,
	0x80, 0x76,
	0x81, 0x80,
	0x82, 0x88,
	0x83, 0x8f,
	0x84, 0x96,
	0x85, 0xa3,
	0x86, 0xaf,
	0x87, 0xc4,
	0x88, 0xd7,
	0x89, 0xe8,
	0x13, 0xe0,
	0x00, 0x00,
	0x10, 0x00,
	0x0d, 0x40,
	0x14, 0x38,
	0xa5, 0x05,
	0xab, 0x07,
	0x24, 0x95,
	0x25, 0x33,
	0x26, 0xe3,
	0x9f, 0x78,
	0xa0, 0x68,
	0xa1, 0x0b,
	0xa6, 0xd8,
	0xa7, 0xd8,
	0xa8, 0xf0,
	0xa9, 0x90,
	0xaa, 0x94,
	0x13, 0xe5,
	0x0e, 0x61,
	0x0f, 0x4b,
	0x16, 0x02,
	0x21, 0x02,
	0x22, 0x91,
	0x29, 0x07,
	0x33, 0x03,
	0x35, 0x0b,
	0x37, 0x1c,
	0x38, 0x71,
	0x3c, 0x78,
	0x4d, 0x40,
	0x4e, 0x20,
	0x69, 0x55,
	0x6b, 0x4a,
	0x74, 0x19,
	0x8d, 0x4f,
	0x8e, 0x00,
	0x8f, 0x00,
	0x90, 0x00,
	0x91, 0x00,
	0x96, 0x00,
	0x9a, 0x80,
	0xb0, 0x8c,
	0xb1, 0x0c,
	0xb2, 0x0e,
	0xb3, 0x82,
	0xb8, 0x0a,
	0x43, 0x14,
	0x44, 0xf0,
	0x45, 0x34,
	0x46, 0x58,
	0x47, 0x28,
	0x48, 0x3a,
	0x59, 0x88,
	0x5a, 0x88,
	0x5b, 0x44,
	0x5c, 0x67,
	0x5d, 0x49,
	0x5e, 0x0e,
	0x6c, 0x0a,
	0x6d, 0x55,
	0x6e, 0x11,
	0x6f, 0x9f,
	0x6a, 0x40,
	0x01, 0x40,
	0x02, 0x40,
	0x13, 0xe7,
	0x4f, 0x80,
	0x50, 0x80,
	0x51, 0x00,
	0x52, 0x22,
	0x53, 0x5e,
	0x54, 0x80,
	0x58, 0x9e,
	0x41, 0x08,
	0x3f, 0x00,
	0x75, 0x05,
	0x76, 0x61,
	0x4c, 0x00,
	0x77, 0x01,
	0x3d, 0xc3,
	0x4b, 0x09,
	0xc9, 0x60,
	0x41, 0x38,
	0x56, 0x40,
	0x34, 0x11,
	0x3b, 0x02,
	0xa4, 0x88,
	0x96, 0x00,
	0x97, 0x30,
	0x98, 0x20,
	0x99, 0x20,
	0x9a, 0x84,
	0x9b, 0x29,
	0x9c, 0x03,
	0x9d, 0x4c,
	0x9e, 0x3f,
	0x78, 0x04,
	0x79, 0x01,
	0xc8, 0xf0,
	0x79, 0x0f,
	0xc8, 0x20,
	0x79, 0x10,
	0xc8, 0x7e,
	0x79, 0x0b,
	0xc8, 0x01,
	0x79, 0x0c,
	0xc8, 0x07,
	0x79, 0x0d,
	0xc8, 0x20,
	0x79, 0x09,
	0xc8, 0x80,
	0x79, 0x02,
	0xc8, 0xc0,
	0x79, 0x03,
	0xc8, 0x40,
	0x79, 0x05,
	0xc8, 0x30,
	0x79, 0x26,
	0x1e, 0x10,
	OV7673_REGEND, 0x00	    /* End of list delimiter */
};

const static u8 gYUV_CIF_5fps_INITIALIZE[] =
{
	0x12, 0x80,
	0x11, 0x81,
	0x92, 0x48,
	0x93, 0x01,
	0x9d, 0x21,
	0x9e, 0x1b,
	0x3b, 0x02,
	0x13, 0xf2,
	0x10, 0x00,
	0x00, 0x00,
	0x01, 0x80,
	0x02, 0x80,
	0x13, 0xf7,
	0x12, 0x20,
	0x04, 0x00,
	0x18, 0x80,
	0x17, 0x28,
	0x32, 0x92,
	0x19, 0x02,
	0x1a, 0x7a,
	0x03, 0x00,
	0x0e, 0x84,
	0x0f, 0x62,
	0x15, 0x02,
	0x16, 0x02,
	0x1b, 0x01,
	0x1e, 0x01,
	0x29, 0x3c,
	0x33, 0x02,
	0x34, 0x00,
	0x35, 0x84,
	0x36, 0x03,
	0x38, 0x13,
	0x39, 0x43,
	0x3a, 0x0c,
	0x3c, 0x6c,
	0x3d, 0x99,
	0x3f, 0x3c,
	0x40, 0xc1,
	0x41, 0x00,
	0x6b, 0x0f,
	0xa1, 0xc8,
	0x69, 0x80,
	0x43, 0xf0,
	0x44, 0x10,
	0x45, 0x78,
	0x46, 0xa8,
	0x47, 0x60,
	0x48, 0x80,
	0x59, 0xba,
	0x5a, 0x9a,
	0x5b, 0x22,
	0x5c, 0xb9,
	0x5d, 0x9b,
	0x5e, 0x10,
	0x5f, 0xe0,
	0x60, 0x85,
	0x61, 0x60,
	0x9f, 0x9d,
	0xa0, 0xe0,
	0x4f, 0x40,
	0x50, 0x43,
	0x51, 0x03,
	0x52, 0x10,
	0x53, 0x28,
	0x54, 0x38,
	0x55, 0x40,
	0x56, 0x40,
	0x57, 0x40,
	0x58, 0x0d,
	0x8b, 0xcc,
	0x8c, 0xcc,
	0x8d, 0xcf,
	0x6c, 0x40,
	0x6d, 0x30,
	0x6e, 0x4b,
	0x6f, 0x60,
	0x70, 0x70,
	0x71, 0x70,
	0x72, 0x70,
	0x73, 0x70,
	0x74, 0x60,
	0x75, 0x60,
	0x76, 0x50,
	0x77, 0x48,
	0x78, 0x3a,
	0x79, 0x2e,
	0x7a, 0x28,
	0x7b, 0x22,
	0x7c, 0x04,
	0x7d, 0x07,
	0x7e, 0x10,
	0x7f, 0x28,
	0x80, 0x36,
	0x81, 0x44,
	0x82, 0x52,
	0x83, 0x60,
	0x84, 0x6c,
	0x85, 0x78,
	0x86, 0x8c,
	0x87, 0x9e,
	0x88, 0xbb,
	0x89, 0xd2,
	0x8a, 0xe6,
	0x14, 0x4e,
	0x24, 0x68,
	0x25, 0x58,
	OV7673_REGEND, 0x00	    /* End of list delimiter */
};

const static u8 gYUV_QCIF_15fps_INITIALIZE[] =
{
	0x12, 0x80,
	0x11, 0x03,
	0x3a, 0x08,
	0x12, 0x00,
	0x17, 0x39,
	0x18, 0x03,
	0x32, 0x92,
	0x19, 0x03,
	0x1a, 0x7b,
	0x03, 0x0a,
	0x0c, 0x0c,
	0x3e, 0x11,
	0x70, 0x3a,
	0x71, 0x35,
	0x72, 0x11,
	0x73, 0xf1,
	0xa2, 0x52,
	0x7a, 0x20,
	0x7b, 0x1c,
	0x7c, 0x28,
	0x7d, 0x3c,
	0x7e, 0x5a,
	0x7f, 0x68,
	0x80, 0x76,
	0x81, 0x80,
	0x82, 0x88,
	0x83, 0x8f,
	0x84, 0x96,
	0x85, 0xa3,
	0x86, 0xaf,
	0x87, 0xc4,
	0x88, 0xd7,
	0x89, 0xe8,
	0x13, 0xe0,
	0x00, 0x00,
	0x10, 0x00,
	0x0d, 0x40,
	0x14, 0x38,
	0xa5, 0x05,
	0xab, 0x07,
	0x24, 0x95,
	0x25, 0x33,
	0x26, 0xe3,
	0x9f, 0x78,
	0xa0, 0x68,
	0xa1, 0x0b,
	0xa6, 0xd8,
	0xa7, 0xd8,
	0xa8, 0xf0,
	0xa9, 0x90,
	0xaa, 0x94,
	0x13, 0xe5,
	0x0e, 0x61,
	0x0f, 0x4b,
	0x16, 0x02,
	0x21, 0x02,
	0x22, 0x91,
	0x29, 0x07,
	0x33, 0x03,
	0x35, 0x0b,
	0x37, 0x1c,
	0x38, 0x71,
	0x3c, 0x78,
	0x4d, 0x40,
	0x4e, 0x20,
	0x69, 0x55,
	0x6b, 0x4a,
	0x74, 0x19,
	0x8d, 0x4f,
	0x8e, 0x00,
	0x8f, 0x00,
	0x90, 0x00,
	0x91, 0x00,
	0x96, 0x00,
	0x9a, 0x80,
	0xb0, 0x8c,
	0xb1, 0x0c,
	0xb2, 0x0e,
	0xb3, 0x82,
	0xb8, 0x0a,
	0x43, 0x14,
	0x44, 0xf0,
	0x45, 0x34,
	0x46, 0x58,
	0x47, 0x28,
	0x48, 0x3a,
	0x59, 0x88,
	0x5a, 0x88,
	0x5b, 0x44,
	0x5c, 0x67,
	0x5d, 0x49,
	0x5e, 0x0e,
	0x6c, 0x0a,
	0x6d, 0x55,
	0x6e, 0x11,
	0x6f, 0x9f,
	0x6a, 0x40,
	0x01, 0x40,
	0x02, 0x40,
	0x13, 0xe7,
	0x4f, 0x80,
	0x50, 0x80,
	0x51, 0x00,
	0x52, 0x22,
	0x53, 0x5e,
	0x54, 0x80,
	0x58, 0x9e,
	0x41, 0x08,
	0x3f, 0x00,
	0x75, 0x05,
	0x76, 0x61,
	0x4c, 0x00,
	0x77, 0x01,
	0x3d, 0xc3,
	0x4b, 0x09,
	0xc9, 0x60,
	0x41, 0x38,
	0x56, 0x40,
	0x34, 0x11,
	0x3b, 0x02,
	0xa4, 0x88,
	0x96, 0x00,
	0x97, 0x30,
	0x98, 0x20,
	0x99, 0x20,
	0x9a, 0x84,
	0x9b, 0x29,
	0x9c, 0x03,
	0x9d, 0x4c,
	0x9e, 0x3f,
	0x78, 0x04,
	0x79, 0x01,
	0xc8, 0xf0,
	0x79, 0x0f,
	0xc8, 0x20,
	0x79, 0x10,
	0xc8, 0x7e,
	0x79, 0x0b,
	0xc8, 0x01,
	0x79, 0x0c,
	0xc8, 0x07,
	0x79, 0x0d,
	0xc8, 0x20,
	0x79, 0x09,
	0xc8, 0x80,
	0x79, 0x02,
	0xc8, 0xc0,
	0x79, 0x03,
	0xc8, 0x40,
	0x79, 0x05,
	0xc8, 0x30,
	0x79, 0x26,
	0x1e, 0x10,
	OV7673_REGEND, 0x00	    /* End of list delimiter */
};

const static u8 gYUV_QCIF_5fps_INITIALIZE[] =
{
	0x12, 0x80,
	0x11, 0x80,
	0x92, 0x68,
	0x93, 0x06,
	0x9d, 0x62,
	0x9e, 0x52,
	0x3b, 0x02,
	0x13, 0xf2,
	0x10, 0x00,
	0x00, 0x00,
	0x01, 0x80,
	0x02, 0x80,
	0x13, 0xf7,
	0x12, 0x08,
	0x04, 0x04,
	0x18, 0x80,
	0x17, 0x28,
	0x32, 0x92,
	0x19, 0x02,
	0x1a, 0x4a,
	0x03, 0x00,
	0x0e, 0x84,
	0x0f, 0x62,
	0x15, 0x02,
	0x16, 0x02,
	0x1b, 0x01,
	0x1e, 0x01,
	0x29, 0x3c,
	0x33, 0x02,
	0x34, 0x00,
	0x35, 0x84,
	0x36, 0x03,
	0x38, 0x13,
	0x39, 0x43,
	0x3a, 0x0c,
	0x3c, 0x6c,
	0x3d, 0x99,
	0x3f, 0x3c,
	0x40, 0xc1,
	0x41, 0x00,
	0x6b, 0x0f,
	0xa1, 0xc8,
	0x69, 0x80,
	0x43, 0xf0,
	0x44, 0x10,
	0x45, 0x78,
	0x46, 0xa8,
	0x47, 0x60,
	0x48, 0x80,
	0x59, 0xba,
	0x5a, 0x9a,
	0x5b, 0x22,
	0x5c, 0xb9,
	0x5d, 0x9b,
	0x5e, 0x10,
	0x5f, 0xe0,
	0x60, 0x85,
	0x61, 0x60,
	0x9f, 0x9d,
	0xa0, 0xe0,
	0x4f, 0x40,
	0x50, 0x43,
	0x51, 0x03,
	0x52, 0x10,
	0x53, 0x28,
	0x54, 0x38,
	0x55, 0x40,
	0x56, 0x40,
	0x57, 0x40,
	0x58, 0x0d,
	0x8b, 0xcc,
	0x8c, 0xcc,
	0x8d, 0xcf,
	0x6c, 0x40,
	0x6d, 0x30,
	0x6e, 0x4b,
	0x6f, 0x60,
	0x70, 0x70,
	0x71, 0x70,
	0x72, 0x70,
	0x73, 0x70,
	0x74, 0x60,
	0x75, 0x60,
	0x76, 0x50,
	0x77, 0x48,
	0x78, 0x3a,
	0x79, 0x2e,
	0x7a, 0x28,
	0x7b, 0x22,
	0x7c, 0x04,
	0x7d, 0x07,
	0x7e, 0x10,
	0x7f, 0x28,
	0x80, 0x36,
	0x81, 0x44,
	0x82, 0x52,
	0x83, 0x60,
	0x84, 0x6c,
	0x85, 0x78,
	0x86, 0x8c,
	0x87, 0x9e,
	0x88, 0xbb,
	0x89, 0xd2,
	0x8a, 0xe6,
	0x14, 0x4e,
	0x24, 0x68,
	0x25, 0x58,
	OV7673_REGEND, 0x00	    /* End of list delimiter */
};

const static u8 gYUV_QQCIF_15fps_INITIALIZE[] =
{
	0x12, 0x80,
	0x11, 0x03,
	0x3a, 0x08,
	0x12, 0x00,
	0x17, 0x3a,
	0x18, 0x04,
	0x32, 0xb6,
	0x19, 0x03,
	0x1a, 0x7b,
	0x03, 0x0a,
	0x0c, 0x0c,
	0x3e, 0x12,
	0x70, 0x3a,
	0x71, 0x35,
	0x72, 0x22,
	0x73, 0xf2,
	0xa2, 0x2a,
	0x7a, 0x20,
	0x7b, 0x1c,
	0x7c, 0x28,
	0x7d, 0x3c,
	0x7e, 0x5a,
	0x7f, 0x68,
	0x80, 0x76,
	0x81, 0x80,
	0x82, 0x88,
	0x83, 0x8f,
	0x84, 0x96,
	0x85, 0xa3,
	0x86, 0xaf,
	0x87, 0xc4,
	0x88, 0xd7,
	0x89, 0xe8,
	0x13, 0xe0,
	0x00, 0x00,
	0x10, 0x00,
	0x0d, 0x40,
	0x14, 0x38,
	0xa5, 0x05,
	0xab, 0x07,
	0x24, 0x95,
	0x25, 0x33,
	0x26, 0xe3,
	0x9f, 0x78,
	0xa0, 0x68,
	0xa1, 0x0b,
	0xa6, 0xd8,
	0xa7, 0xd8,
	0xa8, 0xf0,
	0xa9, 0x90,
	0xaa, 0x94,
	0x13, 0xe5,
	0x0e, 0x61,
	0x0f, 0x4b,
	0x16, 0x02,
	0x21, 0x02,
	0x22, 0x91,
	0x29, 0x07,
	0x33, 0x03,
	0x35, 0x0b,
	0x37, 0x1c,
	0x38, 0x71,
	0x3c, 0x78,
	0x4d, 0x40,
	0x4e, 0x20,
	0x69, 0x55,
	0x6b, 0x4a,
	0x74, 0x19,
	0x8d, 0x4f,
	0x8e, 0x00,
	0x8f, 0x00,
	0x90, 0x00,
	0x91, 0x00,
	0x96, 0x00,
	0x9a, 0x80,
	0xb0, 0x8c,
	0xb1, 0x0c,
	0xb2, 0x0e,
	0xb3, 0x82,
	0xb8, 0x0a,
	0x43, 0x14,
	0x44, 0xf0,
	0x45, 0x34,
	0x46, 0x58,
	0x47, 0x28,
	0x48, 0x3a,
	0x59, 0x88,
	0x5a, 0x88,
	0x5b, 0x44,
	0x5c, 0x67,
	0x5d, 0x49,
	0x5e, 0x0e,
	0x6c, 0x0a,
	0x6d, 0x55,
	0x6e, 0x11,
	0x6f, 0x9f,
	0x6a, 0x40,
	0x01, 0x40,
	0x02, 0x40,
	0x13, 0xe7,
	0x4f, 0x80,
	0x50, 0x80,
	0x51, 0x00,
	0x52, 0x22,
	0x53, 0x5e,
	0x54, 0x80,
	0x58, 0x9e,
	0x41, 0x08,
	0x3f, 0x00,
	0x75, 0x05,
	0x76, 0x61,
	0x4c, 0x00,
	0x77, 0x01,
	0x3d, 0xc3,
	0x4b, 0x09,
	0xc9, 0x60,
	0x41, 0x38,
	0x56, 0x40,
	0x34, 0x11,
	0x3b, 0x02,
	0xa4, 0x88,
	0x96, 0x00,
	0x97, 0x30,
	0x98, 0x20,
	0x99, 0x20,
	0x9a, 0x84,
	0x9b, 0x29,
	0x9c, 0x03,
	0x9d, 0x4c,
	0x9e, 0x3f,
	0x78, 0x04,
	0x79, 0x01,
	0xc8, 0xf0,
	0x79, 0x0f,
	0xc8, 0x20,
	0x79, 0x10,
	0xc8, 0x7e,
	0x79, 0x0b,
	0xc8, 0x01,
	0x79, 0x0c,
	0xc8, 0x07,
	0x79, 0x0d,
	0xc8, 0x20,
	0x79, 0x09,
	0xc8, 0x80,
	0x79, 0x02,
	0xc8, 0xc0,
	0x79, 0x03,
	0xc8, 0x40,
	0x79, 0x05,
	0xc8, 0x30,
	0x79, 0x26,
	0x1e, 0x10,
	OV7673_REGEND, 0x00	    /* End of list delimiter */
};          
            
const static u8 gYUV_QQCIF_5fps_INITIALIZE[] =
{           
	0x12, 0x80,
	0x11, 0x80,
	0x92, 0x68,
	0x93, 0x06,
	0x9d, 0x62,
	0x9e, 0x52,
	0x3b, 0x02,
	0x13, 0xf2,
	0x10, 0x00,
	0x00, 0x00,
	0x01, 0x80,
	0x02, 0x80,
	0x13, 0xf7,
	0x12, 0x08,
	0x04, 0x24,
	0x18, 0x80,
	0x17, 0x28,
	0x32, 0x92,
	0x19, 0x02,
	0x1a, 0x4a,
	0x03, 0x00,
	0x0e, 0x84,
	0x0f, 0x62,
	0x15, 0x02,
	0x16, 0x02,
	0x1b, 0x01,
	0x1e, 0x01,
	0x29, 0x3c,
	0x33, 0x02,
	0x34, 0x00,
	0x35, 0x84,
	0x36, 0x03,
	0x38, 0x13,
	0x39, 0x43,
	0x3a, 0x0c,
	0x3c, 0x6c,
	0x3d, 0x99,
	0x3f, 0x3c,
	0x40, 0xc1,
	0x41, 0x00,
	0x6b, 0x0f,
	0xa1, 0xc8,
	0x69, 0x80,
	0x43, 0xf0,
	0x44, 0x10,
	0x45, 0x78,
	0x46, 0xa8,
	0x47, 0x60,
	0x48, 0x80,
	0x59, 0xba,
	0x5a, 0x9a,
	0x5b, 0x22,
	0x5c, 0xb9,
	0x5d, 0x9b,
	0x5e, 0x10,
	0x5f, 0xe0,
	0x60, 0x85,
	0x61, 0x60,
	0x9f, 0x9d,
	0xa0, 0xe0,
	0x4f, 0x40,
	0x50, 0x43,
	0x51, 0x03,
	0x52, 0x10,
	0x53, 0x28,
	0x54, 0x38,
	0x55, 0x40,
	0x56, 0x40,
	0x57, 0x40,
	0x58, 0x0d,
	0x8b, 0xcc,
	0x8c, 0xcc,
	0x8d, 0xcf,
	0x6c, 0x40,
	0x6d, 0x30,
	0x6e, 0x4b,
	0x6f, 0x60,
	0x70, 0x70,
	0x71, 0x70,
	0x72, 0x70,
	0x73, 0x70,
	0x74, 0x60,
	0x75, 0x60,
	0x76, 0x50,
	0x77, 0x48,
	0x78, 0x3a,
	0x79, 0x2e,
	0x7a, 0x28,
	0x7b, 0x22,
	0x7c, 0x04,
	0x7d, 0x07,
	0x7e, 0x10,
	0x7f, 0x28,
	0x80, 0x36,
	0x81, 0x44,
	0x82, 0x52,
	0x83, 0x60,
	0x84, 0x6c,
	0x85, 0x78,
	0x86, 0x8c,
	0x87, 0x9e,
	0x88, 0xbb,
	0x89, 0xd2,
	0x8a, 0xe6,
	0x14, 0x4e,
	0x24, 0x68,
	0x25, 0x58,
	OV7673_REGEND, 0x00	    /* End of list delimiter */
};

const static u8 gYUV_QQVGA_15fps_INITIALIZE[] =
{
	0x12, 0x80,
	0x11, 0x03,
	0x3a, 0x08,
	0x12, 0x00,
	0x32, 0x80,
	0x17, 0x16,
	0x18, 0x04,
	0x19, 0x02,
	0x1a, 0x7a,
	0x03, 0x0a,
	0x0c, 0x04,
	0x3e, 0x1a,
	0x70, 0x3a,
	0x71, 0x35,
	0x72, 0x22,
	0x73, 0xf2,
	0xa2, 0x02,
	0x7a, 0x20,
	0x7b, 0x1c,
	0x7c, 0x28,
	0x7d, 0x3c,
	0x7e, 0x5a,
	0x7f, 0x68,
	0x80, 0x76,
	0x81, 0x80,
	0x82, 0x88,
	0x83, 0x8f,
	0x84, 0x96,
	0x85, 0xa3,
	0x86, 0xaf,
	0x87, 0xc4,
	0x88, 0xd7,
	0x89, 0xe8,
	0x13, 0xe0,
	0x00, 0x00,
	0x10, 0x00,
	0x0d, 0x40,
	0x14, 0x38,
	0xa5, 0x05,
	0xab, 0x07,
	0x24, 0x95,
	0x25, 0x33,
	0x26, 0xe3,
	0x9f, 0x78,
	0xa0, 0x68,
	0xa1, 0x0b,
	0xa6, 0xd8,
	0xa7, 0xd8,
	0xa8, 0xf0,
	0xa9, 0x90,
	0xaa, 0x94,
	0x13, 0xe5,
	0x0e, 0x61,
	0x0f, 0x4b,
	0x16, 0x02,
	0x21, 0x02,
	0x22, 0x91,
	0x29, 0x07,
	0x33, 0x03,
	0x35, 0x0b,
	0x37, 0x1c,
	0x38, 0x71,
	0x3c, 0x78,
	0x4d, 0x40,
	0x4e, 0x20,
	0x69, 0x55,
	0x6b, 0x4a,
	0x74, 0x19,
	0x8d, 0x4f,
	0x8e, 0x00,
	0x8f, 0x00,
	0x90, 0x00,
	0x91, 0x00,
	0x96, 0x00,
	0x9a, 0x80,
	0xb0, 0x8c,
	0xb1, 0x0c,
	0xb2, 0x0e,
	0xb3, 0x82,
	0xb8, 0x0a,
	0x43, 0x14,
	0x44, 0xf0,
	0x45, 0x34,
	0x46, 0x58,
	0x47, 0x28,
	0x48, 0x3a,
	0x59, 0x88,
	0x5a, 0x88,
	0x5b, 0x44,
	0x5c, 0x67,
	0x5d, 0x49,
	0x5e, 0x0e,
	0x6c, 0x0a,
	0x6d, 0x55,
	0x6e, 0x11,
	0x6f, 0x9f,
	0x6a, 0x40,
	0x01, 0x40,
	0x02, 0x40,
	0x13, 0xe7,
	0x4f, 0x80,
	0x50, 0x80,
	0x51, 0x00,
	0x52, 0x22,
	0x53, 0x5e,
	0x54, 0x80,
	0x58, 0x9e,
	0x41, 0x08,
	0x3f, 0x00,
	0x75, 0x05,
	0x76, 0x61,
	0x4c, 0x00,
	0x77, 0x01,
	0x3d, 0xc3,
	0x4b, 0x09,
	0xc9, 0x60,
	0x41, 0x38,
	0x56, 0x40,
	0x34, 0x11,
	0x3b, 0x02,
	0xa4, 0x88,
	0x96, 0x00,
	0x97, 0x30,
	0x98, 0x20,
	0x99, 0x20,
	0x9a, 0x84,
	0x9b, 0x29,
	0x9c, 0x03,
	0x9d, 0x4c,
	0x9e, 0x3f,
	0x78, 0x04,
	0x79, 0x01,
	0xc8, 0xf0,
	0x79, 0x0f,
	0xc8, 0x20,
	0x79, 0x10,
	0xc8, 0x7e,
	0x79, 0x0b,
	0xc8, 0x01,
	0x79, 0x0c,
	0xc8, 0x07,
	0x79, 0x0d,
	0xc8, 0x20,
	0x79, 0x09,
	0xc8, 0x80,
	0x79, 0x02,
	0xc8, 0xc0,
	0x79, 0x03,
	0xc8, 0x40,
	0x79, 0x05,
	0xc8, 0x30,
	0x79, 0x26,
	0x1e, 0x10,
	OV7673_REGEND, 0x00	    /* End of list delimiter */
};

const static u8 gYUV_QQVGA_5fps_INITIALIZE[] =
{
	0x12, 0x80,
	0x11, 0x81,
	0x92, 0xd8,
	0x93, 0x03,
	0x9d, 0x41,
	0x9e, 0x36,
	0x3b, 0x02,
	0x13, 0xf2,
	0x10, 0x00,
	0x00, 0x00,
	0x01, 0x80,
	0x02, 0x80,
	0x13, 0xf7,
	0x12, 0x10,
	0x04, 0x24,
	0x18, 0x4b,
	0x17, 0x23,
	0x32, 0xbf,
	0x19, 0x02,
	0x1a, 0x7a,
	0x03, 0x00,
	0x0e, 0x84,
	0x0f, 0x62,
	0x15, 0x02,
	0x16, 0x02,
	0x1b, 0x01,
	0x1e, 0x01,
	0x29, 0x3c,
	0x33, 0x00,
	0x34, 0x07,
	0x35, 0x84,
	0x36, 0x00,
	0x38, 0x13,
	0x39, 0x43,
	0x3a, 0x0c,
	0x3c, 0x6c,
	0x3d, 0x99,
	0x3f, 0x3c,
	0x40, 0xc1,
	0x41, 0x00,
	0x6b, 0x0a,
	0xa1, 0xc8,
	0x69, 0x80,
	0x43, 0xf0,
	0x44, 0x10,
	0x45, 0x78,
	0x46, 0xa8,
	0x47, 0x60,
	0x48, 0x80,
	0x59, 0xba,
	0x5a, 0x9a,
	0x5b, 0x22,
	0x5c, 0xb9,
	0x5d, 0x9b,
	0x5e, 0x10,
	0x5f, 0xe0,
	0x60, 0x85,
	0x61, 0x60,
	0x9f, 0x9d,
	0xa0, 0xe0,
	0x4f, 0x40,
	0x50, 0x43,
	0x51, 0x03,
	0x52, 0x10,
	0x53, 0x28,
	0x54, 0x38,
	0x55, 0x40,
	0x56, 0x40,
	0x57, 0x40,
	0x58, 0x0d,
	0x8b, 0xcc,
	0x8c, 0xcc,
	0x8d, 0xcf,
	0x6c, 0x40,
	0x6d, 0x30,
	0x6e, 0x4b,
	0x6f, 0x60,
	0x70, 0x70,
	0x71, 0x70,
	0x72, 0x70,
	0x73, 0x70,
	0x74, 0x60,
	0x75, 0x60,
	0x76, 0x50,
	0x77, 0x48,
	0x78, 0x3a,
	0x79, 0x2e,
	0x7a, 0x28,
	0x7b, 0x22,
	0x7c, 0x04,
	0x7d, 0x07,
	0x7e, 0x10,
	0x7f, 0x28,
	0x80, 0x36,
	0x81, 0x44,
	0x82, 0x52,
	0x83, 0x60,
	0x84, 0x6c,
	0x85, 0x78,
	0x86, 0x8c,
	0x87, 0x9e,
	0x88, 0xbb,
	0x89, 0xd2,
	0x8a, 0xe6,
	0x14, 0x4e,
	0x24, 0x68,
	0x25, 0x58,
	OV7673_REGEND, 0x00	    /* End of list delimiter */
};

const static u8 gYUV_QVGA_15fps_INITIALIZE[] =
{
	0x12, 0x80,
	0x11, 0x03,
	0x3a, 0x08,
	0x12, 0x00,
	0x32, 0x80,
	0x17, 0x16,
	0x18, 0x04,
	0x19, 0x02,
	0x1a, 0x7a,
	0x03, 0x0a,
	0x0c, 0x04,
	0x3e, 0x19,
	0x70, 0x3a,
	0x71, 0x35,
	0x72, 0x11,
	0x73, 0xf1,
	0xa2, 0x02,
	0x7a, 0x20,
	0x7b, 0x1c,
	0x7c, 0x28,
	0x7d, 0x3c,
	0x7e, 0x5a,
	0x7f, 0x68,
	0x80, 0x76,
	0x81, 0x80,
	0x82, 0x88,
	0x83, 0x8f,
	0x84, 0x96,
	0x85, 0xa3,
	0x86, 0xaf,
	0x87, 0xc4,
	0x88, 0xd7,
	0x89, 0xe8,
	0x13, 0xe0,
	0x00, 0x00,
	0x10, 0x00,
	0x0d, 0x40,
	0x14, 0x38,
	0xa5, 0x05,
	0xab, 0x07,
	0x24, 0x95,
	0x25, 0x33,
	0x26, 0xe3,
	0x9f, 0x78,
	0xa0, 0x68,
	0xa1, 0x0b,
	0xa6, 0xd8,
	0xa7, 0xd8,
	0xa8, 0xf0,
	0xa9, 0x90,
	0xaa, 0x94,
	0x13, 0xe5,
	0x0e, 0x61,
	0x0f, 0x4b,
	0x16, 0x02,
	0x21, 0x02,
	0x22, 0x91,
	0x29, 0x07,
	0x33, 0x03,
	0x35, 0x0b,
	0x37, 0x1c,
	0x38, 0x71,
	0x3c, 0x78,
	0x4d, 0x40,
	0x4e, 0x20,
	0x69, 0x55,
	0x6b, 0x4a,
	0x74, 0x19,
	0x8d, 0x4f,
	0x8e, 0x00,
	0x8f, 0x00,
	0x90, 0x00,
	0x91, 0x00,
	0x96, 0x00,
	0x9a, 0x80,
	0xb0, 0x8c,
	0xb1, 0x0c,
	0xb2, 0x0e,
	0xb3, 0x82,
	0xb8, 0x0a,
	0x43, 0x14,
	0x44, 0xf0,
	0x45, 0x34,
	0x46, 0x58,
	0x47, 0x28,
	0x48, 0x3a,
	0x59, 0x88,
	0x5a, 0x88,
	0x5b, 0x44,
	0x5c, 0x67,
	0x5d, 0x49,
	0x5e, 0x0e,
	0x6c, 0x0a,
	0x6d, 0x55,
	0x6e, 0x11,
	0x6f, 0x9f,
	0x6a, 0x40,
	0x01, 0x40,
	0x02, 0x40,
	0x13, 0xe7,
	0x4f, 0x80,
	0x50, 0x80,
	0x51, 0x00,
	0x52, 0x22,
	0x53, 0x5e,
	0x54, 0x80,
	0x58, 0x9e,
	0x41, 0x08,
	0x3f, 0x00,
	0x75, 0x05,
	0x76, 0x61,
	0x4c, 0x00,
	0x77, 0x01,
	0x3d, 0xc1,
	0x4b, 0x09,
	0xc9, 0x60,
	0x41, 0x38,
	0x56, 0x40,
	0x34, 0x11,
	0x3b, 0x02,
	0xa4, 0x88,
	0x96, 0x00,
	0x97, 0x30,
	0x98, 0x20,
	0x99, 0x20,
	0x9a, 0x84,
	0x9b, 0x29,
	0x9c, 0x03,
	0x9d, 0x4c,
	0x9e, 0x3f,
	0x78, 0x04,
	0x79, 0x01,
	0xc8, 0xf0,
	0x79, 0x0f,
	0xc8, 0x20,
	0x79, 0x10,
	0xc8, 0x7e,
	0x79, 0x0b,
	0xc8, 0x01,
	0x79, 0x0c,
	0xc8, 0x07,
	0x79, 0x0d,
	0xc8, 0x20,
	0x79, 0x09,
	0xc8, 0x80,
	0x79, 0x02,
	0xc8, 0xc0,
	0x79, 0x03,
	0xc8, 0x40,
	0x79, 0x05,
	0xc8, 0x30,
	0x79, 0x26,
	0x1e, 0x10,
	OV7673_REGEND, 0x00	    /* End of list delimiter */
};

const static u8 gYUV_QVGA_5fps_INITIALIZE[] =
{
	0x12, 0x80,
	0x11, 0x81,
	0x92, 0x48,
	0x93, 0x01,
	0x9d, 0x21,
	0x9e, 0x1b,
	0x3b, 0x02,
	0x13, 0xf2,
	0x10, 0x00,
	0x00, 0x00,
	0x01, 0x80,
	0x02, 0x80,
	0x13, 0xf7,
	0x12, 0x10,
	0x04, 0x00,
	0x18, 0x4b,
	0x17, 0x23,
	0x32, 0xbf,
	0x19, 0x02,
	0x1a, 0x7a,
	0x03, 0x00,
	0x0e, 0x84,
	0x0f, 0x62,
	0x15, 0x02,
	0x16, 0x02,
	0x1b, 0x01,
	0x1e, 0x01,
	0x29, 0x3c,
	0x33, 0x00,
	0x34, 0x07,
	0x35, 0x84,
	0x36, 0x00,
	0x38, 0x13,
	0x39, 0x43,
	0x3a, 0x0c,
	0x3c, 0x6c,
	0x3d, 0x99,
	0x3f, 0x3c,
	0x40, 0xc1,
	0x41, 0x00,
	0x6b, 0x0a,
	0xa1, 0xc8,
	0x69, 0x80,
	0x43, 0xf0,
	0x44, 0x10,
	0x45, 0x78,
	0x46, 0xa8,
	0x47, 0x60,
	0x48, 0x80,
	0x59, 0xba,
	0x5a, 0x9a,
	0x5b, 0x22,
	0x5c, 0xb9,
	0x5d, 0x9b,
	0x5e, 0x10,
	0x5f, 0xe0,
	0x60, 0x85,
	0x61, 0x60,
	0x9f, 0x9d,
	0xa0, 0xe0,
	0x4f, 0x40,
	0x50, 0x43,
	0x51, 0x03,
	0x52, 0x10,
	0x53, 0x28,
	0x54, 0x38,
	0x55, 0x40,
	0x56, 0x40,
	0x57, 0x40,
	0x58, 0x0d,
	0x8b, 0xcc,
	0x8c, 0xcc,
	0x8d, 0xcf,
	0x6c, 0x40,
	0x6d, 0x30,
	0x6e, 0x4b,
	0x6f, 0x60,
	0x70, 0x70,
	0x71, 0x70,
	0x72, 0x70,
	0x73, 0x70,
	0x74, 0x60,
	0x75, 0x60,
	0x76, 0x50,
	0x77, 0x48,
	0x78, 0x3a,
	0x79, 0x2e,
	0x7a, 0x28,
	0x7b, 0x22,
	0x7c, 0x04,
	0x7d, 0x07,
	0x7e, 0x10,
	0x7f, 0x28,
	0x80, 0x36,
	0x81, 0x44,
	0x82, 0x52,
	0x83, 0x60,
	0x84, 0x6c,
	0x85, 0x78,
	0x86, 0x8c,
	0x87, 0x9e,
	0x88, 0xbb,
	0x89, 0xd2,
	0x8a, 0xe6,
	0x14, 0x4e,
	0x24, 0x68,
	0x25, 0x58,
	OV7673_REGEND, 0x00	    /* End of list delimiter */
};

const static u8 gYUV_VGA_15fps_INITIALIZE[] =
{
	0x12, 0x80,
	0x11, 0x03,
	0x3a, 0x08,
	0x12, 0x00,
	0x17, 0x13,
	0x18, 0x01,
	0x32, 0xb6,
	0x19, 0x02,
	0x1a, 0x7a,
	0x03, 0x0a,
	0x0c, 0x00,
	0x3e, 0x00,
	0x70, 0x3a,
	0x71, 0x35,
	0x72, 0x11,
	0x73, 0xf0,
	0xa2, 0x02,
	0x7a, 0x20,
	0x7b, 0x1c,
	0x7c, 0x28,
	0x7d, 0x3c,
	0x7e, 0x5a,
	0x7f, 0x68,
	0x80, 0x76,
	0x81, 0x80,
	0x82, 0x88,
	0x83, 0x8f,
	0x84, 0x96,
	0x85, 0xa3,
	0x86, 0xaf,
	0x87, 0xc4,
	0x88, 0xd7,
	0x89, 0xe8,
	0x13, 0xe0,
	0x00, 0x00,
	0x10, 0x00,
	0x0d, 0x40,
	0x14, 0x38,
	0xa5, 0x05,
	0xab, 0x07,
	0x24, 0x95,
	0x25, 0x33,
	0x26, 0xe3,
	0x9f, 0x78,
	0xa0, 0x68,
	0xa1, 0x0b,
	0xa6, 0xd8,
	0xa7, 0xd8,
	0xa8, 0xf0,
	0xa9, 0x90,
	0xaa, 0x94,
	0x13, 0xe5,
	0x0e, 0x61,
	0x0f, 0x4b,
	0x16, 0x02,
	0x21, 0x02,
	0x22, 0x91,
	0x29, 0x07,
	0x33, 0x03,
	0x35, 0x0b,
	0x37, 0x1c,
	0x38, 0x71,
	0x3c, 0x78,
	0x4d, 0x40,
	0x4e, 0x20,
	0x69, 0x55,
	0x6b, 0x4a,
	0x74, 0x19,
	0x8d, 0x4f,
	0x8e, 0x00,
	0x8f, 0x00,
	0x90, 0x00,
	0x91, 0x00,
	0x96, 0x00,
	0x9a, 0x80,
	0xb0, 0x8c,
	0xb1, 0x0c,
	0xb2, 0x0e,
	0xb3, 0x82,
	0xb8, 0x0a,
	0x43, 0x14,
	0x44, 0xf0,
	0x45, 0x34,
	0x46, 0x58,
	0x47, 0x28,
	0x48, 0x3a,
	0x59, 0x88,
	0x5a, 0x88,
	0x5b, 0x44,
	0x5c, 0x67,
	0x5d, 0x49,
	0x5e, 0x0e,
	0x6c, 0x0a,
	0x6d, 0x55,
	0x6e, 0x11,
	0x6f, 0x9f,
	0x6a, 0x40,
	0x01, 0x40,
	0x02, 0x40,
	0x13, 0xe7,
	0x4f, 0x80,
	0x50, 0x80,
	0x51, 0x00,
	0x52, 0x22,
	0x53, 0x5e,
	0x54, 0x80,
	0x58, 0x9e,
	0x41, 0x08,
	0x3f, 0x00,
	0x75, 0x05,
	0x76, 0x61,
	0x4c, 0x00,
	0x77, 0x01,
	0x3d, 0xc1,
	0x4b, 0x09,
	0xc9, 0x60,
	0x41, 0x38,
	0x56, 0x40,
	0x34, 0x11,
	0x3b, 0x02,
	0xa4, 0x88,
	0x96, 0x00,
	0x97, 0x30,
	0x98, 0x20,
	0x99, 0x20,
	0x9a, 0x84,
	0x9b, 0x29,
	0x9c, 0x03,
	0x9d, 0x4c,
	0x9e, 0x3f,
	0x78, 0x04,
	0x79, 0x01,
	0xc8, 0xf0,
	0x79, 0x0f,
	0xc8, 0x20,
	0x79, 0x10,
	0xc8, 0x7e,
	0x79, 0x0b,
	0xc8, 0x01,
	0x79, 0x0c,
	0xc8, 0x07,
	0x79, 0x0d,
	0xc8, 0x20,
	0x79, 0x09,
	0xc8, 0x80,
	0x79, 0x02,
	0xc8, 0xc0,
	0x79, 0x03,
	0xc8, 0x40,
	0x79, 0x05,
	0xc8, 0x30,
	0x79, 0x26,
	0x1e, 0x10,
	OV7673_REGEND, 0x00	    /* End of list delimiter */
};

const static u8 gYUV_VGA_5fps_INITIALIZE[] =
{
	0x12, 0x80,
	0x11, 0x81,
	0x92, 0x48,
	0x93, 0x01,
	0x9d, 0x19,
	0x9e, 0x15,
	0x3b, 0x02,
	0x13, 0xf2,
	0x10, 0x00,
	0x00, 0x00,
	0x01, 0x80,
	0x02, 0x80,
	0x13, 0xf7,
	0x12, 0x00,
	0x04, 0x00,
	0x18, 0x01,
	0x17, 0x13,
	0x32, 0x92,
	0x19, 0x02,
	0x1a, 0x7a,
	0x03, 0x00,
	0x0e, 0x84,
	0x0f, 0x62,
	0x15, 0x02,
	0x16, 0x02,
	0x1b, 0x01,
	0x1e, 0x01,
	0x29, 0x3c,
	0x33, 0x00,
	0x34, 0x07,
	0x35, 0x84,
	0x36, 0x00,
	0x38, 0x13,
	0x39, 0x43,
	0x3a, 0x0c,
	0x3c, 0x6c,
	0x3d, 0x99,
	0x3f, 0x3c,
	0x40, 0xc1,
	0x41, 0x00,
	0x6b, 0x0a,
	0xa1, 0xc8,
	0x69, 0x80,
	0x43, 0xf0,
	0x44, 0x10,
	0x45, 0x78,
	0x46, 0xa8,
	0x47, 0x60,
	0x48, 0x80,
	0x59, 0xba,
	0x5a, 0x9a,
	0x5b, 0x22,
	0x5c, 0xb9,
	0x5d, 0x9b,
	0x5e, 0x10,
	0x5f, 0xe0,
	0x60, 0x85,
	0x61, 0x60,
	0x9f, 0x9d,
	0xa0, 0xe0,
	0x4f, 0x40,
	0x50, 0x43,
	0x51, 0x03,
	0x52, 0x10,
	0x53, 0x28,
	0x54, 0x38,
	0x55, 0x40,
	0x56, 0x40,
	0x57, 0x40,
	0x58, 0x0d,
	0x8b, 0xcc,
	0x8c, 0xcc,
	0x8d, 0xcf,
	0x6c, 0xf0,
	0x6d, 0xf0,
	0x6e, 0xf0,
	0x6f, 0x70,
	0x70, 0x58,
	0x71, 0x58,
	0x72, 0x48,
	0x73, 0x48,
	0x74, 0x38,
	0x75, 0x40,
	0x76, 0x34,
	0x77, 0x34,
	0x78, 0x2e,
	0x79, 0x28,
	0x7a, 0x24,
	0x7b, 0x22,
	0x7c, 0x0f,
	0x7d, 0x1e,
	0x7e, 0x3c,
	0x7f, 0x58,
	0x80, 0x63,
	0x81, 0x6e,
	0x82, 0x77,
	0x83, 0x80,
	0x84, 0x87,
	0x85, 0x8f,
	0x86, 0x9c,
	0x87, 0xa9,
	0x88, 0xc0,
	0x89, 0xd4,
	0x8a, 0xe6,
	0x14, 0x4e,
	0x24, 0x68,
	0x25, 0x58,
	OV7673_REGEND, 0x00	    /* End of list delimiter */
};

const static u8 Premier_Lens[] =
{
	OV7673_AWBTH3   , 0x5C,
	OV7673_AWBTH4   , 0x6e,
	OV7673_AWBTH5   , 0x46,
	OV7673_AWBTH6   , 0x4D,
	OV7673_AWBC1    , 0x23,
	OV7673_AWBC2    , 0x41,
	OV7673_AWBC3    , 0x43,
	OV7673_AWBC4    , 0x74,
	OV7673_AWBC5    , 0x6e,

	OV7673_REGEND, 	0x00		/* End of list delimiter */
};

const static u8 LiteOn_Lens[] =
{
	OV7673_AWBTH3   , 0x58,
	OV7673_AWBTH4   , 0x6c,
	OV7673_AWBTH5   , 0x46,
	OV7673_AWBTH6   , 0x48,
	OV7673_AWBC1    , 0x23,
	OV7673_AWBC2    , 0x41,
	OV7673_AWBC3    , 0x54,
	OV7673_AWBC4    , 0x74,
	OV7673_AWBC5    , 0x5c,

	OV7673_REGEND, 	0x00		/* End of list delimiter */
};

const static u8 gSensorSlaveAddr = 0x21;
static int read_sensor_reg( const u8 subAddress, u8 *bufP );
static int write_sensor_reg( const u8 subAddress, u8 *bufP );

struct OV7673_SETTING
{
	int   format;
	int   resolution;
	const u8 *setting;
};

struct OV7673_SETTING ov7673_setting_table[] =
{
#if	0
	{OV_FORMAT_RAW8   , OV_SIZE_QQVGA, gRAW_QQVGA_INITIALIZE},
	{OV_FORMAT_RAW8   , OV_SIZE_QVGA , gRAW_QVGA_INITIALIZE},
	{OV_FORMAT_RAW8   , OV_SIZE_VGA  , gRAW_VGA_INITIALIZE},
	{OV_FORMAT_RAW8   , OV_SIZE_QQCIF, gRAW_QQCIF_INITIALIZE},
	{OV_FORMAT_RAW8   , OV_SIZE_QCIF , gRAW_QCIF_INITIALIZE},
	{OV_FORMAT_RAW8   , OV_SIZE_CIF  , gRAW_CIF_INITIALIZE},
#endif
	{OV_FORMAT_YUV_422, OV_SIZE_QVGA , gYUV_QVGA_15fps_INITIALIZE},
	{OV_FORMAT_YUV_422, OV_SIZE_VGA  , gYUV_VGA_15fps_INITIALIZE},
	{OV_FORMAT_YUV_422, OV_SIZE_QQVGA, gYUV_QQVGA_15fps_INITIALIZE},
	{OV_FORMAT_YUV_422, OV_SIZE_QQCIF, gYUV_QQCIF_15fps_INITIALIZE},
	{OV_FORMAT_YUV_422, OV_SIZE_QCIF , gYUV_QCIF_15fps_INITIALIZE},
	{OV_FORMAT_YUV_422, OV_SIZE_CIF  , gYUV_CIF_15fps_INITIALIZE},
	{OV_FORMAT_RGB_565, OV_SIZE_QQVGA, gRGB565_QQVGA_15fps_INITIALIZE},
	{OV_FORMAT_RGB_565, OV_SIZE_QVGA , gRGB565_QVGA_15fps_INITIALIZE},
	{OV_FORMAT_RGB_565, OV_SIZE_VGA  , gRGB565_VGA_15fps_INITIALIZE},
	{OV_FORMAT_RGB_565, OV_SIZE_QQCIF, gRGB565_QQCIF_15fps_INITIALIZE},
	{OV_FORMAT_RGB_565, OV_SIZE_QCIF , gRGB565_QCIF_15fps_INITIALIZE},
	{OV_FORMAT_RGB_565, OV_SIZE_CIF  , gRGB565_CIF_15fps_INITIALIZE},
	{0, 0, 0}
} ;

/*****************************************************************************
 *  Private/helper api
 *****************************************************************************/
#ifdef DEBUG_PARAM_CHECK
static int get_reg_value( u8 *regP, u8 regAddr, u8 *regValueP )
{
	unsigned int index = 0;
	u8 curReg = 0;

	while( curReg < OV7673_REGEND )
	{
		curReg = regP[index << 1];
		if( curReg == regAddr )
		{
			*regValueP = regP[(index << 1) + 1];
			return 0;
		}
		index ++;
	}

	return -EIO;

}

static int set_reg_value( u8 *regP, u8 regAddr, u8 regValue )
{
	unsigned int index = 0;
	u8 curReg = 0;

	while( curReg < OV7673_REGEND )
	{
		curReg = regP[index << 1];
		if( curReg == regAddr )
		{
			regP[(index << 1) + 1] = regValue;
			return 0;
		}
		index ++;
	}

	return -EIO;

}
#endif

/*****************************************************************************
 *  Sensor read/write
 *****************************************************************************/
#if 0
extern int pxa_i2c_write(u8 slaveaddr, const u8 *bytebuf, u32 bytescount);
extern int pxa_i2c_read(u8 slaveaddr, u8 *bytesbuf, u32 bytescount);

static int read_sensor_reg( const u8 subAddress, u8 *bufP )
{
	u8 buffer[1];
	int status;
	buffer[0] = subAddress;

	status = pxa_i2c_write( gSensorSlaveAddr, buffer, 1);
	if (!status) {
		status = pxa_i2c_read( gSensorSlaveAddr, buffer, 1);
		*bufP = buffer[0];
	}
	return status;
}

static int write_sensor_reg( const u8 subAddress, u8 *bufP )
{
	u8 buffer[2];
	int status;
	buffer[0] = subAddress;
	buffer[1] = *bufP;

	status = pxa_i2c_write( gSensorSlaveAddr, buffer, 2);
	return status;
}
#endif
static int rmw_sensor_reg( const u8 subAddress,
		u8 *bufP, u8 andMask, u8 orMask )
{
	int status;
	status = read_sensor_reg( subAddress, bufP );
	if (!status) {
		*bufP &= andMask;
		*bufP |= orMask;
		status = write_sensor_reg( subAddress, bufP );
	}
	return status;
}

int ov7673hw_read_sensor_reg( const u8 subAddress, u8 *bufP )
{
	return read_sensor_reg(subAddress, bufP);
}

int ov7673hw_write_sensor_reg( const u8 subAddress, u8 *bufP )
{
	return write_sensor_reg(subAddress, bufP);
}

int ov7673hw_set_regs( const u8 *regP )
{
	u32	curReg = 0;
	int	status = 0;

	/* The list is a register number followed by the value */
	while( regP[curReg << 1] < OV7673_REGEND )
	{
		u8 regVal = regP[(curReg << 1) + 1];

		status = (write_sensor_reg( regP[curReg << 1], &regVal ) == 0) ?
			0 : -EIO;

		if( curReg == 0 )
			ov7673hw_wait( 5 );

		curReg++;
	}

	return status;
}

int ov7673hw_read_all_regs( u8 *bufP, u32 numRegs )
{
	u32	curReg;

	for( curReg = 0; curReg < numRegs; curReg++, bufP++ )
		read_sensor_reg( (u8)curReg, bufP );


	return 0;
}

#ifdef CONFIG_MACH_ZYLONITE
static void sensor_power_on(void)
{
	mhn_gpio_set_direction(MFP_CIF_LO_PWDN_GPI0, GPIO_DIR_OUT);
	mhn_gpio_set_level(MFP_CIF_LO_PWDN_GPI0, GPIO_LEVEL_LOW);
}

static void sensor_power_off(void)
{
	mhn_gpio_set_level(MFP_CIF_LO_PWDN_GPI0, GPIO_LEVEL_HIGH);
	mhn_gpio_set_direction(MFP_CIF_LO_PWDN_GPI0, GPIO_DIR_IN);
}
#elif CONFIG_MACH_LITTLETON
extern int max7320_read(void);
extern int max7320_write(u8 value);
static void sensor_power_on(void)
{
	int val;
	if (platform_id() == 0x0) { /* Littleton 1.x */
		val = max7320_read();
		val &= ~0x20;
		max7320_write(val);
	}
}

static void sensor_power_off(void)
{
	int val;
	if (platform_id() == 0x0) { /* Littleton 1.x */
		val = max7320_read();
		val |= 0x20;
		max7320_write(val);
	}
}
#endif

/*****************************************************************************
 *  Power & Reset
 *****************************************************************************/
void ov7673hw_power_down( u8 powerMode )
{
	/* OV7673 PWRDWN, 0 = NORMAL, 1=POWER DOWN */
	if( powerMode == CAMERA_POWER_OFF )
		sensor_power_off();
	else
		sensor_power_on();

	mdelay(100);
}

void ov7673hw_reset( )
{
	return;
}

void ov7673hw_wait( int ms )
{
	mdelay( ms );
}


/*****************************************************************************
 *  Settings
 *****************************************************************************/
int ov7673hw_version_revision(u8 * pCmRevision, u8 *pSensorRevision)
{
	read_sensor_reg( OV7673_PID, pCmRevision );
	read_sensor_reg( OV7673_VER, pSensorRevision );
	return 0;
}

void ov7673hw_set_hsync()
{
	u8	val;

	/* Makes HREF become HSYNC */
	read_sensor_reg( OV7673_COM10, &val );
	val |= 0x40;
	write_sensor_reg( OV7673_COM10, &val );
}

void ov7673hw_auto_function_on()
{
	u8 val;
	read_sensor_reg( OV7673_COM8, &val );
	val |= 0x07;	/* don't disturb AWB */
	write_sensor_reg( OV7673_COM8, &val );
}

void ov7673hw_auto_function_off()
{
	u8 val;
	read_sensor_reg( OV7673_COM8, &val );
	val &= ~0x07;	/* don't disturb AWB */
	write_sensor_reg( OV7673_COM8, &val );
}


/*****************************************************************************
 *  Viewfinder, still
 *****************************************************************************/
int ov7673hw_viewfinder_on()
{
	u8	com3;

	read_sensor_reg( OV7673_COM3, &com3 );
	com3 &= ~0x01;
	write_sensor_reg( OV7673_COM3, &com3 );

	return OV_ERR_NONE;
}


int ov7673hw_viewfinder_off()
{
	u8	com3;

	read_sensor_reg( OV7673_COM3, &com3 );
	com3 |= 0x01;
	write_sensor_reg( OV7673_COM3, &com3 );

	return OV_ERR_NONE;
}


int ov7673hw_halt_video_output()
{
	u8	com3;

	/* Set the camera to only output 1 frame */
	read_sensor_reg( OV7673_COM3, &com3 );
	com3 |= 1;
	write_sensor_reg( OV7673_COM3, &com3 );

	return OV_ERR_NONE;
}

int ov7673hw_resumeto_full_output_mode()
{
	u8	mode;

	/* Output still frames continuously
	 * Turn off single capture mode COM3.
	 */
	rmw_sensor_reg( OV7673_COM3, (&mode), ((u8) ~1), 0 );
	return OV_ERR_NONE;
}

int ov7673hw_get_single_image()
{
	u8	mode;

	rmw_sensor_reg( OV7673_COM3, &mode, (u8) ~1, 1 );
	return OV_ERR_NONE;
}

#define MIN(x,y) ((x < y) ? x : y)
#define MAX(x,y) ((x > y) ? x : y)

/*****************************************************************************
 *  Format
 *****************************************************************************/
int ov7673hw_set_format( u32 captureSizeFormat, u32 colorFormat, u32 mode)
{

	int	status;
	u8	*regsP, regValue;
	const u8	*defaultDataP;
	struct OV7673_SETTING  *ov7673_setting;

	defaultDataP  = NULL;
	for(ov7673_setting = ov7673_setting_table;
		ov7673_setting->format != 0;ov7673_setting++) {
		if ((colorFormat == ov7673_setting->format) &&
			(captureSizeFormat == ov7673_setting->resolution)) {
			defaultDataP = ov7673_setting->setting;
			break;
		}
	}

	if (defaultDataP == NULL)
		return -EPERM;

	/* reset */
	regValue = 0x80;
	write_sensor_reg(OV7673_COM7, &regValue);

	/* Get the pointer to the basic setting.
	 * The pointer must be freed after exiting.
	 */
	regsP = (u8 *) defaultDataP;

	/* Blast the entire parameter tree into the part */
	status = ov7673hw_set_regs( regsP );

	return 0;
}

/*****************************************************************************
 * Contrast
 *****************************************************************************/
const static u8 ContrastLowestSettings[] = {
	0x6C, 0x80,
	0x6D, 0xa0,
	0x6E, 0x78,
	0x6F, 0x50,
	0x70, 0x48,
	0x71, 0x40,
	0x72, 0x48,
	0x73, 0x40,
	0x74, 0x40,
	0x75, 0x40,
	0x76, 0x40,
	0x77, 0x40,
	0x78, 0x3e,
	0x79, 0x3c,
	0x7A, 0x3c,
	0x7B, 0x28,
	0x7C, 0x8,
	0x7D, 0x12,
	0x7E, 0x21,
	0x7F, 0x35,
	0x80, 0x3e,
	0x81, 0x46,
	0x82, 0x4f,
	0x83, 0x57,
	0x84, 0x5f,
	0x85, 0x67,
	0x86, 0x77,
	0x87, 0x87,
	0x88, 0xa6,
	0x89, 0xc4,
	0x8A, 0xe2,
	OV7673_REGEND, 	0x00		/* End of list delimiter */
};

const static u8 ContrastLowSettings[] = {
	0x6C, 0x70,
	0x6D, 0x80,
	0x6E, 0x40,
	0x6F, 0x54,
	0x70, 0x58,
	0x71, 0x60,
	0x72, 0x60,
	0x73, 0x60,
	0x74, 0x50,
	0x75, 0x58,
	0x76, 0x44,
	0x77, 0x3c,
	0x78, 0x30,
	0x79, 0x28,
	0x7A, 0x22,
	0x7B, 0x44,
	0x7C, 0x7,
	0x7D, 0xf,
	0x7E, 0x17,
	0x7F, 0x2c,
	0x80, 0x37,
	0x81, 0x43,
	0x82, 0x4f,
	0x83, 0x5b,
	0x84, 0x65,
	0x85, 0x70,
	0x86, 0x81,
	0x87, 0x90,
	0x88, 0xa8,
	0x89, 0xbc,
	0x8A, 0xcd,
	OV7673_REGEND, 	0x00		/* End of list delimiter */
};

const static u8 ContrastMiddleSettings[] = {
	0x6C, 0x40,
	0x6D, 0x30,
	0x6E, 0x4B,
	0x6F, 0x60,
	0x70, 0x70,
	0x71, 0x70,
	0x72, 0x70,
	0x73, 0x70,
	0x74, 0x60,
	0x75, 0x60,
	0x76, 0x50,
	0x77, 0x48,
	0x78, 0x3A,
	0x79, 0x2E,
	0x7A, 0x28,
	0x7B, 0x22,
	0x7C, 0x4,
	0x7D, 0x7,
	0x7E, 0x10,
	0x7F, 0x28,
	0x80, 0x36,
	0x81, 0x44,
	0x82, 0x52,
	0x83, 0x60,
	0x84, 0x6C,
	0x85, 0x78,
	0x86, 0x8C,
	0x87, 0x9E,
	0x88, 0xBB,
	0x89, 0xD2,
	0x8A, 0xE6,
	OV7673_REGEND, 	0x00		/* End of list delimiter */
};

const static u8 ContrastHighSettings[] = {
	0x6c, 0x10,
	0x6d, 0x10,
	0x6e, 0x18,
	0x6f, 0x5c,
	0x70, 0x90,
	0x71, 0x90,
	0x72, 0x90,
	0x73, 0x90,
	0x74, 0x80,
	0x75, 0x80,
	0x76, 0x60,
	0x77, 0x5c,
	0x78, 0x44,
	0x79, 0x24,
	0x7a, 0x1a,
	0x7b, 0x10,
	0x7c, 0x1,
	0x7d, 0x2,
	0x7e, 0x5,
	0x7f, 0x1c,
	0x80, 0x2e,
	0x81, 0x40,
	0x82, 0x52,
	0x83, 0x64,
	0x84, 0x74,
	0x85, 0x84,
	0x86, 0x9c,
	0x87, 0xb3,
	0x88, 0xd5,
	0x89, 0xe7,
	0x8a, 0xf4,
	OV7673_REGEND, 	0x00		/* End of list delimiter */
};

const static u8 ContrastHighestSettings[] = {
	0x6c, 0x20,
	0x6d, 0x40,
	0x6e, 0x10,
	0x6f, 0x38,
	0x70, 0x80,
	0x71, 0xe0,
	0x72, 0xd0,
	0x73, 0xe8,
	0x74, 0xa0,
	0x75, 0x80,
	0x76, 0x80,
	0x77, 0x54,
	0x78, 0x30,
	0x79, 0x22,
	0x7a, 0x8,
	0x7b, 0x1,
	0x7c, 0x2,
	0x7d, 0x6,
	0x7e, 0x8,
	0x7f, 0x16,
	0x80, 0x26,
	0x81, 0x42,
	0x82, 0x5c,
	0x83, 0x79,
	0x84, 0x8d,
	0x85, 0x9d,
	0x86, 0xbd,
	0x87, 0xd2,
	0x88, 0xea,
	0x89, 0xfb,
	0x8a, 0xff,
	OV7673_REGEND, 	0x00		/* End of list delimiter */
};

int ov7673hw_set_contrast(u32 value)
{
	const u8 *regP;

	regP = NULL;
	switch(value) {
		case SENSOR_CONTRAST_LOWEST:
			regP = ContrastLowestSettings;
			break;
		case SENSOR_CONTRAST_LOW:
			regP = ContrastLowSettings;
			break;
		case SENSOR_CONTRAST_MIDDLE:
			regP = ContrastMiddleSettings;
			break;
		case SENSOR_CONTRAST_HIGH:
			regP = ContrastHighSettings;
			break;
		case SENSOR_CONTRAST_HIGHEST:
			regP = ContrastHighestSettings;
			break;
		default:
			regP = ContrastMiddleSettings;
			break;
	}

	/* set hw */
	if (regP)
		ov7673hw_set_regs(regP);
	return 0;
}

/*****************************************************************************
 * Exposure
 *****************************************************************************/
const static u8 ExposureSettings[] = {
	0x40, 0x30, 0x81, /* EV-2 */
	0x58, 0x48, 0x91, /* EV-1 */
	0x88, 0x7c, 0x93, /* EV0 */
	0xa0, 0x90, 0xb4, /* EV+1 */
	0xc0, 0xb0, 0xd6, /* EV+2 */
};

int ov7673hw_set_exposure(u32 value)
{
	u8 aew, aeb, vpt;
	int index = -1;

	switch(value) {
		case SENSOR_EXPOSURE_LOWEST:
			index = 0;
			break;
		case SENSOR_EXPOSURE_LOW:
			index = 3;
			break;
		case SENSOR_EXPOSURE_MIDDLE:
			index = 6;
			break;
		case SENSOR_EXPOSURE_HIGH:
			index = 9;
			break;
		case SENSOR_EXPOSURE_HIGHEST:
			index = 12;
			break;
		default:
			break;
	}

	aew = aeb = vpt = 0;

	if (index != -1){
		aew = ExposureSettings[index];
		aeb = ExposureSettings[index + 1];
		vpt  = ExposureSettings[index + 2];
	}

	/* set hw */
	if (aew || aeb || vpt)
	{
		ov7673hw_write_sensor_reg( 0x24, &aew);
		ov7673hw_write_sensor_reg( 0x25, &aeb);
		ov7673hw_write_sensor_reg( 0x26, &vpt);
	}

	return 0;
}

/*****************************************************************************
 * Auto White Balance
 *****************************************************************************/
const static u8 AWBAuto[] = {
	0x13, 0xad,
	0x01, 0x80,
	0x02, 0x80,
	0x60, 0x14,
	0x5f, 0x05,
	0x13, 0xaf,
	OV7673_REGEND, 	0x00		/* End of list delimiter */
};

const static u8 AWBFluorescent[] = {
	0x13, 0xad,
	0x01, 0x6c,
	0x02, 0x2e,
	0x5f, 0x05,
	OV7673_REGEND, 	0x00		/* End of list delimiter */
};

const static u8 AWBOutdoor[] = {
	0x13, 0xad,
	0x01, 0x44,
	0x02, 0x44,
	0x5f, 0x05,
	OV7673_REGEND, 	0x00		/* End of list delimiter */
};

const static u8 AWBIncandescent[] = {
	0x13, 0xad,
	0x01, 0x6c,
	0x02, 0x20,
	0x5f, 0x05,
	OV7673_REGEND, 	0x00		/* End of list delimiter */
};


int ov7673hw_set_white_balance(u32 value)
{
	const u8 *regP;

	regP = NULL;
	switch(value) {
		case SENSOR_WHITEBALANCE_AUTO:		/* Auto */
			regP = AWBAuto;
			break;
		case SENSOR_WHITEBALANCE_INCANDESCENT:	/* Incandescent */
			regP = AWBIncandescent;
			break;
		case SENSOR_WHITEBALANCE_SUNNY:		/* Sunny */
			regP = AWBOutdoor;
			break;
		case SENSOR_WHITEBALANCE_FLUORESCENT:	/* Fluorescent */
			regP = AWBFluorescent;
			break;
		default:
			break;
	}

	/* set hw */
	if (regP) {
		ov7673hw_set_regs(regP);
	}
	return 0;
}

/*****************************************************************************
 *                              OV7673 I2C Client Driver
 *****************************************************************************/
#include <linux/i2c.h>
static int i2c_ov7673_attach_adapter(struct i2c_adapter *adapter);
static int i2c_ov7673_detect_client(struct i2c_adapter *, int, int);
static int i2c_ov7673_detach_client(struct i2c_client *client);
#define I2C_DRIVERID_OV7673	(0)
#define	OV7673_ADDRESS	0x21

struct i2c_driver ov7673_driver  =
{
	.driver		= {
		.name	= "ov7673 i2c client driver",
	},
	.id		= I2C_DRIVERID_OV7673,
	.attach_adapter	= &i2c_ov7673_attach_adapter,
	.detach_client	= &i2c_ov7673_detach_client,
};

/* Unique ID allocation */
static struct i2c_client *g_client;
static unsigned short normal_i2c[] = {OV7673_ADDRESS, I2C_CLIENT_END };
I2C_CLIENT_INSMOD_1(ov7673);

static int read_sensor_reg( const u8 subAddress, u8 *bufP )
{
	int ret;

	if( g_client == NULL )	/*	No global client pointer?	*/
		return -1;

	ret = i2c_smbus_read_byte_data(g_client, subAddress);
	if (ret >= 0) {
		*bufP = ret;
	}
	return ret;
}

static int write_sensor_reg( const u8 subAddress, u8 *bufP )
{
	if( g_client == NULL )	/*	No global client pointer?	*/
		return -1;

	return i2c_smbus_write_byte_data(g_client, subAddress, *bufP);
}

static int i2c_ov7673_read(struct i2c_client *client, u8 reg)
{
	return i2c_smbus_read_byte_data(client,reg);
}

static int i2c_ov7673_attach_adapter(struct i2c_adapter *adap)
{
	return i2c_probe(adap,&addr_data,i2c_ov7673_detect_client);
}

static int i2c_ov7673_detect_client(struct i2c_adapter *adapter,
		int address, int kind)
{
	struct i2c_client *new_client;
	int err = 0;

	/* Let's see whether this adapter can support what we need.
	 * Please substitute the things you need here!
	 */
	if ( !i2c_check_functionality(adapter,I2C_FUNC_SMBUS_BYTE_DATA) ) {
		printk(KERN_INFO "byte op is not permited.\n");
		goto ERROR0;
	}

	/* OK. For now, we presume we have a valid client. We now create the
	 * client structure, even though we cannot fill it completely yet.
	 * But it allows us to access several i2c functions safely
	 */

	/* Note that we reserve some space for ov7673_data too. If you don't
	 * need it, remove it. We do it here to help to lessen memory
	 * fragmentation.
	 */

	new_client = kmalloc(sizeof(struct i2c_client), GFP_KERNEL );

	if ( !new_client )  {
		err = -ENOMEM;
		printk(KERN_ALERT "Failed to allocate i2c client.\n");
		goto ERROR0;
	}

	/* new_client->addr = address; */
	/* FIXME */
	new_client->addr = address;
	new_client->adapter = adapter;
	new_client->driver = &ov7673_driver;
//	new_client->flags = 0;

	/* detect OV7673 */
	pxa_set_cken(CKEN_CAMERA, 1);
	pxa3xx_mfp_set_afds(MFP_CIF_MCLK, MFP_AF0, MFP_DS04X);
	ci_set_clock(1, 1, 1300);
	sensor_power_on();
	mdelay(1);

	if ((i2c_ov7673_read(new_client, OV7673_PID) != PID_OV76XX ) ||
			(i2c_ov7673_read(new_client, OV7673_VER) != PID_7673 )){
		ci_set_clock(0, 0, 1300);
		sensor_power_off();
		printk(KERN_INFO "OV7673 not detected.\n");
		pxa_set_cken(CKEN_CAMERA, 0);
		goto ERROR1;
	} else {
		extern int ov7673_detected;
		ov7673_detected = 1;
		printk(KERN_INFO "OV7673 detected.\n");
	}
	ci_set_clock(0, 0, 1300);
	sensor_power_off();
	pxa_set_cken(CKEN_CAMERA, 0);

	g_client = new_client;

	strcpy(new_client->name, "OV7673");

	/* Tell the i2c layer a new client has arrived */
	if ((err = i2c_attach_client(new_client)))
		goto ERROR1;

	return 0;

ERROR1:
	kfree(new_client);
ERROR0:
	return err;
}

static int i2c_ov7673_detach_client(struct i2c_client *client)
{
	int err;

	/* Try to detach the client from i2c space */
	if ((err = i2c_detach_client(client))) {
		return err;
	}

	/* Frees client data too, if allocated at the same time */
	kfree(client);
	g_client = NULL;
	return 0;
}

static int __init i2c_ov7673_init(void)
{
	int ret;

	if ( (ret = i2c_add_driver(&ov7673_driver)) ) {
		printk(KERN_ERR "ov7673: Driver registration failed,"
			"module not inserted.\n");
		return ret;
	}

	return 0;
}

static void __exit i2c_ov7673_exit(void)
{
	if (i2c_del_driver(&ov7673_driver)) {
		printk(KERN_ERR "ov7673: Driver registration failed,"
		"module not removed.\n");
	}
}

MODULE_DESCRIPTION("OV7673 I2C Client driver");
MODULE_LICENSE("GPL");

module_init(i2c_ov7673_init);
module_exit(i2c_ov7673_exit);
